package Modules::HomeStructure;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

#
# Modules::HomeStructure - the module to operate with system of catalogues
#                          and privileges of home's dir
#

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

use Modules::File::TreeEntry;

@ISA = ('Exporter');
@EXPORT = qw/ &fixGroup &fixOwner /;

my ($homeTree,$lastError,$lastErrorCode,$lastErrorText,$lastFullPath,$extrUserName);
my ($apacheUid,$apacheGid,$userUid,$userGid);

my  $class="Modules::HomeStructure";

{
	my @homeDir = ('home','r','r',0755,'d');
	my @homeContent = (['.configs','r','r',0700,'d'],
										 ['atd','r','a',0750,'d'],
										 ['html','u','a',0750,'d',
											['cgi-bin','u','u',0755,'d']],
										 ['phptmp','u','a',0770,'d'],
										 ['files','u','a',0750,'d'],
										 ['log','r','u',0750,'d',
											['access_log','r','r',0644,'f'],
										 ],
										 ['restore','r','u',0770,'d'],
										 ['backup','r','u',0750,'d',
											['html.tar.gz','r','u',0640,'f'],
											['files.tar.gz','r','u',0640,'f'],
											['mysql.tar.gz','r','u',0640,'f'],
										 ],
										 ['.forward','u','u',0640,'f']
									);


	$homeTree = Modules::File::TreeEntry->new('name'=>$homeDir[0],'owner'=>$homeDir[1],
																						'group'=>$homeDir[2],'mode'=>$homeDir[3],
																						'type'=>'d');

	my ($newEntry,$i,$cnt, $ptrArr);
	foreach $ptrArr (@homeContent) {
		$newEntry = $homeTree->addToContent('name'=>$ptrArr->[0],'owner'=>$ptrArr->[1],
																				'group'=>$ptrArr->[2],'mode'=>$ptrArr->[3],
																				'type'=>$ptrArr->[4]||'d');
		$cnt = @{$ptrArr};
		for ( $i = 5; $i < $cnt; $i++){
			if(ref($ptrArr->[$i])=~/ARRAY/){
				my @entry = @{$ptrArr->[$i]};
				$newEntry->addToContent('name'=>$entry[0],'owner'=>$entry[1],
																'group'=>$entry[2],'mode'=>$entry[3],
																'type'=>$entry[4]||'d');
			}
		}
	}

	$homeTree->mapOwner('r',0,'r',0);
}



sub new {
	my $className = shift;
	
	my %args = ( 'user'=>undef,
							 'group'=>undef,
							 'uid'=>undef,
							 'gid'=>undef,
							 'home'=>undef,
							 'apacheGroup'=>$::apacheGroup,
							 'apacheUser'=>$::apacheUser,
							 'apacheGid'=>$::gidapache,
							 @_);
	
	my $this = {'user'=>$args{'user'}||$args{'name'}||$args{'owner'},
							'group'=>$args{'group'},
							'uid'=>$args{'uid'},
							'gid'=>$args{'gid'}||$args{'uid'},
							'apacheGroup'=>$args{'apacheGroup'},
							'apacheUser'=>$args{'apacheUser'},
							'apacheGid'=>$args{'apacheGid'},
						 };
	$this->{'group'} = $this->{'user'} unless $this->{'group'};
	$this->{'restore_users_permissions'} = exists($args{'restore_users_permissions'})?
		$args{'restore_users_permissions'}:1;

	bless ($this,$className);

	if ( $args{'apacheUser'} ) {
		$this->Apache( $args{'apacheUser'}, $args{'apacheGroup'} );
	}
	if ( exists( $args{'gidapache'} ) && $args{'gidapache'} =~ /^\d+$/ ) {
		$this->{'apacheGid'} = ($args{'gidapache'});
	}		

	$apacheUid = $this->{'apacheUid'};
	$apacheGid = $this->{'apacheGid'};

						
	if($args{'home'}){
		$this->{'homeDir'} = $args{'home'};
	}else{
		if ( $args{'user'} ) {
			$this->{'homeDir'} = $::user_homeDir.'/'.$args{'user'};
		}
	}
		
	$homeTree->mapOwner( 'a', $this->{'apacheUid'}, 'a', $this->{'apacheGid'} );
	$homeTree->Base( $::user_homeDir );

	$extrUserName = qr/^\Q$::user_homeDir\E\/([^\/]+)/;

  return $this;
}

#
# Set/Get user's info (name, uid, gid)
#
# argumetns: 
#             hash with keys ('user','group','uid','gid')
#
# return: 
#            pointer to anonymous hash with sam keys  
# 
sub User {
	my $this = shift;
	
	if ( @_ ) {
		my %args = (@_);
		my $user = $args{'user'}||$args{'name'}||$args{'owner'};
		if($user){
			$this->{'user'} = $user;
			$this->{'group'} = $args{'group'}? $args{'group'}: $args{'user'};
			if ( exists($args{'uid'}) && $args{'uid'} =~ /^\d+$/ ) {
				$this->{'uid'} = $args{'uid'};
				$this->{'gid'} = $args{'gid'} =~ /^\d+$/? $args{'gid'}: $args{'uid'};
			}else{
				$this->setUserInfo();
			}

			$homeTree->Name( $user );
			$homeTree->mapOwner( 'u', $this->{'uid'}, 'u', $this->{'gid'} );

		}
	}

	$userUid = $this->{'uid'};
	$userGid = $this->{'gid'};

	if(wantarray()){
		return ('user'=>$this->{'user'},
						'group'=>$this->{'group'},
						'uid'=>$this->{'uid'},
						'gid'=>$this->{'gid'},
					 );
	}else{
		return $this->{'user'};
	}
}

#
# fill user's info by name
#
sub setUserInfo {
	my $this=shift;

	if (@_){
		$this->{'user'}=shift;
	}
	my (@usersInfo);
	my $user = $this->{'user'};

	unless( @usersInfo = getpwnam($user)){
		return -1;
	}

	($this->{'uid'},$this->{'gid'},$this->{'homeDir'})=@usersInfo[2,3,7];

	$userUid = $this->{'uid'};
	$userGid = $this->{'gid'};

	$this->{'group'}=getgrgid($this->{'gid'});

	return $this->{'uid'};
}

sub Apache {
	my $this = shift;
	if(@_){
		my ($name,$group)  = @_;
		my (@info,@infoGroup);
		if ( @info = getpwnam($name) ) {
			$this->{'apacheUser'} = $name;
			$this->{'apacheUid'} = $info[2];
			if ( $group && ( @infoGroup = getgrnam($group) ) ) {
			  $this->{'apacheGid'} = $infoGroup[2];
			  $this->{'apacheGroup'} = $group;
			} else {
  			  $this->{'apacheGid'} = $info[3];
  			  $this->{'apacheGroup'} = getgrgid($this->{'apacheGid'});			  
			}
			
			$apacheUid = $this->{'apacheUid'};
			$apacheGid = $this->{'apacheGid'};

			$homeTree->mapOwner('a',$this->{'apacheUid'},'a',$this->{'apacheGid'});

		}
	}
	if ( wantarray() ) {
		return ($this->{'apacheUid'},$this->{'apacheGid'});
	} else {
		return {'apacheGroup'=>$this->{'apacheGroup'},
					'apacheUser'=>$this->{'apacheUser'},
					'apacheGid'=>$this->{'apacheGid'},
					'apacheUid'=>$this->{'apacheUid'},
				 };
	}
}

sub createHome {
	my $this = shift;
	if(@_){
		$this->User(@_);
	}
	unless ($this->{'user'}){
		return undef;
	}
	unless ($this->{'uid'}=~/^\d+$/ && $this->{'gid'}=~/^\d+$/){
		$this->setUserInfo();
	}

	$userUid = $this->{'uid'};
	$userGid = $this->{'gid'};

	unless( $userUid=~/^\d+$/ && $userGid=~/^\d+$/ ){
		return undef;
	}

	$homeTree->Name($this->{'user'});

	$homeTree->mapOwner('u',$userUid,'u',$userGid);
	$homeTree->RestoreUsersPermissions($this->{'restore_users_permissions'});

	my ( $ret );
	if ( exists ( $this->{'create'} ) ) {
		$ret = $homeTree->create( $this->{'create'} );
		delete $this->{'create'};
	} else {
		$ret = $homeTree->create();
	}
	if($ret){
		return $homeTree->FullPath();
	}else{
		$lastError = $homeTree->lastError();
		($lastErrorCode,$lastErrorText) = $homeTree->lastErrorCode();
		$lastFullPath = $homeTree->lastFullPath();
		return undef;
	}
}

sub repairHome {
	my $this = shift;
	$this->{'create'} = 1;
	$this->createHome(@_);
}

sub Ownership {
	my $this = shift;
	my $fullPath = $_[0];
	if ( $fullPath =~ /$extrUserName/ ) { ## in $1 must be name of user
		my $user = $1;

		if ( $this->User() ne $user ) {
			$this->User( 'name'=>$user );
		}
		if ($homeTree->Name() ne $user){
			$homeTree->Name($user);
		}
		
		if ($homeTree->Base() ne $::user_homeDir){
			$homeTree->Base($::user_homeDir);
		}
	}

	my($uid,$gid,$mode) = $homeTree->Ownership( @_ );

	if(wantarray()){
		return ($uid,$gid,$mode);
	}else{
		return $uid;
	}
}

#
# sub to fix GID during File::Find::find 
#
sub fixGroup {
	my ($realUid,$realGid,$hUid,$hGid,$newGid);
  if ( -l $File::Find::name ) { ## do not work with symbolic links
		return;
	}
	unless ( ($realUid,$realGid) = (stat($File::Find::name))[4,5] ){
		return;
	}
	($hUid,$hGid) = $homeTree->Ownership($File::Find::name);
	if ( defined($hGid) ) {
		$newGid = $hGid;
	} else {
		if ( $realUid == $userUid ) {
			$newGid = $userGid;
		} elsif ( $realUid == $apacheUid ) {
			$newGid = $apacheGid;
		}
	}

	if ( defined( $newGid ) && ( $newGid != $realGid ) ) {
		chown ($realUid, $newGid, $File::Find::name) or
			print STDERR "Error: unable to chown '$File::Find::name': $!\n";
	}
}

#
# sub to fix owner during File::Find::find 
#
sub fixOwner {

	my ($realUid,$realGid,$hUid,$hGid,$newGid, $newUid);

  if ( -l $File::Find::name ) { ## do not work with symbolic links
		return;
	}
	unless ( ($realUid,$realGid) = (stat($File::Find::name))[4,5] ){
		return;
	}

	($hUid,$hGid) = $homeTree->Ownership($File::Find::name); ## get fixed owner

	if ( defined( $hGid ) ) {
		$newGid = $hGid;
	}

	if ( defined( $hUid ) ) {
		$newUid = $hUid;

	} else {
		if ( $realUid == $apacheUid ) { ## owner is apache
			unless ( ( $realGid == $apacheGid ) || ( $realGid == $userGid ) ) {
				$newGid = $apacheGid;
			}

		} elsif ( $realUid != 0) { # owner is not root
			unless ( $realUid == $userUid ) {
				$newUid = $userUid;
			}
			unless ( ( $realGid == $userGid ) || ( $realGid == $apacheGid ) ) {
				$newGid = $userGid;
			}
		}
	}

	if ( defined( $newUid ) || defined( $newGid ) ) {

		chown ( defined( $newUid )? $newUid: $realUid,
					  defined( $newGid )? $newGid: $realGid,
					  $File::Find::name ) or
							print STDERR "Error: unable to chown '$File::Find::name': $!\n";
	}
}



sub Base {
	my $this = shift;
	if(@_){
		$homeTree->Base(@_);
	}
	return $homeTree->Base();
}

sub lastError {
	return $lastError;
}

sub lastErrorCode {
	if ( wantarray() ) {
		return ( $lastErrorCode,$lastErrorText );
	}else{
		return $lastErrorCode;
	}
}

sub lastFullPath {
	return $lastFullPath;
}

sub RestoreUsersPermissions {
	my $this = shift;
	if ( @_ ) {
		$this->{'restore_users_permissions'} = shift;
	}
	return $this->{'restore_users_permissions'};
}

sub makeDir {
	my $this = shift;
	my $fullPath = shift;

	my ($base, $path, $chunk, $dirPath, $uid, $gid, $mode );

	$base = $homeTree->Base;
	unless ( $fullPath =~ /^\Q$base\E\/(.*)/ ) {
		return 0;
	}
	$path = $1;

	unless ( -d $base ) {
		return 0;
	}

	foreach $chunk ( split( /\//, $path ) ) {
		$base .= '/'.$chunk;

		next if ( -d $base );

		($uid,$gid,$mode) = $homeTree->Ownership( $base );
		unless ( $mode ) {
			return 0; ## not found
		}

		unless (  mkdir( $base, $mode ) &&
							chown( $uid, $gid, $base ) &&
							chmod( $mode, $base ) ) {
			return 0; ## error: can not create dir
		}
	}

	return 1; ## success
}

1;
