package Modules::Restore::DumpConfXML;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

# The module is intended to parse xml config (confixx_dump.conf)
# and to store resulting data about tables format in hash/array
# of objects.

# typical usage
# 	$obj_conf = Modules::Restore::DumpConfXML->new('confixx_dump.xml')
# 	@tables = $obj_conf->getTablesList();
#   $obj_tab_conf_1 = $obj_conf->getTable($tables[i]);
#   $obj_tab_conf_2 = $obj_conf->getTable("my_table");

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;
use XML::DOM;
use locale;

@ISA = qw(Exporter);

use Modules::Restore::Table;
use Modules::Restore::Service;
use Modules::Backup::Section;
use Modules::Backup::Dir;


my $error = '';

sub new {
  my $classname = shift;
  my ($config,$oldMainConf) = @_;
	
  my $this = {
	      'tables'	=> {},		# hash of Modules::Restore::Table objects
	      'ord_tables'	=> [],	# array of strings. table names (in order of appearance in config file)
	      'services'=>{},
	      'ord_services' => [],
	      'files'=>{},
	      'ord_files'=>[],
	      'oldMainConf'=>'',
	      'map'=>undef,
	     };
  if (-T $oldMainConf){
    $this->{'oldMainConf'}=$oldMainConf;
  };
  my ($xml_conf,$parser);
  if (ref($config)=~/XML/){
    $xml_conf=$config;
  }else{
    $parser = new XML::DOM::Parser;
    $xml_conf = $parser->parsefile($config);
    $this->{'source'}=$config;
  }	
  my ($conf_main,$kid);
  foreach $kid ($xml_conf->getChildNodes()){
    if ($kid->getNodeName() eq 'confixx'){
      $conf_main=$kid;
      last;
    }
  }
  unless (defined($conf_main)) {
    &errmsg("The XML config is malformed: no 'confixx' node found\n");
    return undef;
  }
  my $kidName;
  my %level1=('data'=>undef, 'service_set'=>undef, 'files'=>undef);
  foreach $kid ($conf_main->getChildNodes()){
    next unless ($kid->getNodeType==ELEMENT_NODE);
    $kidName=$kid->getNodeName();
    if (exists($level1{$kidName})){
      $level1{$kidName} = $kid;  
    }
  }

  my $conf_tables=$level1{'data'}->getChildNodes();

  unless (defined($conf_tables)) {
    &errmsg("The XML config $config is malformed: no 'data' node found\n");
    return undef;
  }
  my $n_tables    = $conf_tables->getLength();
  my ($attr_name,$attr_dump,$attr_open,$attr_id,$conf_table,$operator,$table,
      @open_attrs,$val,$oldAttr);
  my $dbIsPg = ($::dbType eq 'Pg');
  for (my $i = 0; $i < $n_tables; $i++) {
    $conf_table = $conf_tables->item($i);
    next unless ($conf_table->getNodeType==ELEMENT_NODE);
    next unless  ($conf_table->getNodeName() eq 'table'); 
			# detect manipulator to dump object
			# default manipulator is mnp_hash2xmlnode	
    $attr_name = $conf_table->getAttribute('name');	
    if (exists($$this{'tables'}{$attr_name})){
      print STDERR "Error of config parsing: there is already table '$attr_name'. Skiped\n";
      next;
    }
    $attr_dump = $conf_table->getAttribute('dump');	
    $attr_open = $conf_table->getAttribute('safe');	

    if($dbIsPg){
      $attr_open = &::toLowCase($attr_open,$conf_table,'safe');
    }
			
    $attr_dump = 'trivial' if ("$attr_dump" eq ''); 
    $operator  = 'mnp_'.$attr_dump;
			
    if (exists($this->{'tables'}->{$attr_name})){
      print STDERR "WARNING: There is already table '$attr_name'\n";
      $table = $this->{'tables'}->{$attr_name};
    }else{
      $table = Modules::Restore::Table->new($attr_name, $operator);

      $this->{'tables'}->{$attr_name} = $table;
      push @{$$this{'ord_tables'}}, $attr_name;
    }
    @open_attrs = split /,\s*/, $attr_open;
#      my %props;
    foreach my $prop (@open_attrs) {
      $table->addProp($prop, 'safe' => 1)
    }
    $table->fillAttributes($conf_table);
    $table->fillFields($conf_table);

    if($attr_id=$conf_table->getAttribute('id')){
      if($dbIsPg){
	$attr_id=&::toLowCase($attr_id,$conf_table,'id');
      }
      unless ($table->AttrID($attr_id)){
	&errmsg("Table '$attr_name' has not field '$attr_id'. Attribut 'id' is skiped\n");
      }
    }
    if($val=$conf_table->getAttribute('order')){
      if($dbIsPg){
	$val=&::toLowCase($val,$conf_table,'order');
      }
      unless ($table->AttrOrder($val)){
	&errmsg("Table '$attr_name' has not field '$val'. Attribut 'order' is skiped\n");
      }
    }

    $val=$conf_table->getAttribute('noclean')||
      $conf_table->getAttribute('unclean');

    $table->AttrNoClean($val);

  }

  my ($objService,$name);

  if($oldMainConf){
    my $xmlServices=$level1{'service_set'};
  
    if ($xmlServices){
      foreach my $xmlService ($xmlServices->getChildNodes()){
				next unless ($xmlService->getNodeType==ELEMENT_NODE);
				$name=$xmlService->getNodeName();
				next unless  ($name eq 'service'); 
				if($name=$xmlService->getAttribute('name')){
					
					if (exists($$this{'services'}{$name})){
						print STDERR "Error of config parsing: there is already service '$name'. Skiped\n";
						next;
					}
					if ($objService=Modules::Restore::Service->new($xmlService)){
						$$this{'services'}{$name} = $objService;
						push @{$$this{'ord_services'}}, $name;
					}
				}
      }
    }
  }

  my $xmlFiles=$level1{'files'};
  if ($xmlFiles){
    my ($base,$each,$section,$secName,$xmlSection,$path,$key,$value,$dir,$exclude,$kidDir,$delete);
    my @attrs=('type','key','separator','noempty');
    foreach $xmlSection ($xmlFiles->getChildNodes()){
      next unless ($xmlSection->getNodeType==ELEMENT_NODE);
      $name=$xmlSection->getNodeName();
      next unless $name eq 'section';
      $secName=$xmlSection->getAttribute('name');
			
      if (exists($$this{'files'}{$secName})){
				print STDERR "Error of config parsing: there is already section '$secName'. Skiped\n";
				next;
      }
			
      $base=$xmlSection->getAttribute('base');
      next unless ($secName && $base);
      $each=$xmlSection->getAttribute('each');
      $section=	Modules::Backup::Section->new(
																							'name'=> $secName, 
																							'base'=>$base,
																							'each'=>$each
																						 );
      $name=$xmlSection->getAttribute('service');
      if ($name && exists($$this{'services'}{$name})){
				$section->Attribute('service',$name);
      }
      if($name=$xmlSection->getAttribute('rule')){
				$section->Attribute('rule',$name);
      }
      if($value=$xmlSection->getAttribute('if')){
				$section->Attribute('if',$value);
      }
      
      next unless $section;
      $exclude=0;
      $delete=0;
      foreach $kid ($xmlSection->getChildNodes()){
				next unless ($kid->getNodeType==ELEMENT_NODE);
				$name=$kid->getNodeName();
				$path=$kid->getAttribute('name');
				if ($name eq 'dir'){
					if($dir=$section->addDir($path)){
						foreach $key (@attrs){ 
							next unless($value=$kid->getAttribute($key));
							$dir->Attribute($path,$key,$value);
						}
						foreach $kidDir ($kid->getChildNodes()){
							next unless ($kidDir->getNodeType==ELEMENT_NODE);
							$name=$kidDir->getNodeName();
							if($name eq 'exclude'){
								if($path=$kidDir->getAttribute('path')){
									$value=$kidDir->getAttribute('where');
									$dir->addExclusion('path'=>$path,'where'=>$value);
									$exclude=1;
								}
							}elsif($name eq 'delete'){
								if($path=$kidDir->getAttribute('path')){
									$value=$kidDir->getAttribute('where');
									$dir->addDelete('path'=>$path,'where'=>$value);
									$delete=1;
								}
							}
						}
					}
				}elsif($name eq 'file'){
					$section->addFile($path);
					foreach $key (@attrs){ 
						next unless($value=$kid->getAttribute($key));
						$section->FileAttribute($path,$key,$value);
						if (($key eq 'type') and ($value =~ /append/)){
							$section->Attribute('append',1);
						}
					}
				}else{
					next;
				}
				if ($path =~ /$each/){
					$section->Attribute('each',1);
				}
      }
      if($exclude){
				$section->Attribute('exclude',1);
      }
      if($delete){
				$section->Attribute('delete',1);
      }
			
      $$this{'files'}{$secName}=$section;
      push @{$$this{'ord_files'}}, $secName;
    }
  }

  if (ref($parser)=~/XML/ ){
    $xml_conf->dispose;
  }
  
  return bless $this;
}

# return specified array
sub getTable($) {
  my $this = shift;
  my ($table_name) = @_;
  return $$this{'tables'}{$table_name};
}

# return array of tables name
sub getTablesList() {
  my $this = shift;
  return @{ $$this{'ord_tables'} };
}

sub getService($) {
  my $this = shift;
  my ($name) = @_;
  return $$this{'services'}{$name};
}


  # return array of tables name
  
sub getServicesList() {
  my $this = shift;
  return @{ $$this{'ord_services'} };
}

sub getSection($){
  my $this = shift;
  my ($name) = @_;
  return $$this{'files'}{$name};
}

sub getSectionsList(){
  my $this = shift;
  return @{ $$this{'ord_files'} };
}

sub errmsg {
  my ($arg_error) = @_;
  $error = $arg_error;
  warn ("$error\n");
  return 1;
}

sub oldMainConf {
  my $this = shift;
  if (@_){
    my $path = shift;
    if (-T $path){
      $this->{'oldMainConf'}=$path;
    }
  }
  return $this->{'oldMainConf'};
}

sub Map {
  my $this = shift;
  if (@_){
    my $map=shift;
    if(ref($map)=~/Map/){
      $this->{'map'}=$map;
    }
  }
  return $this->{'map'};
}

sub sqlLimit {
    my $this = shift;
  if (@_){
    my $limit=shift;
    if($limit=~/\d+/ && $limit>0){
      $this->{'sql-limit'}=$limit;
    }
  }
  return $this->{'sql-limit'};
}

sub Source {
  my $this = shift;
  if(@_){
    my $path=shift;
    if(-T $path){
      $this->{'source'}=$path;
    }
  }
  return $this->{'source'};
}

1;
