package Modules::Restore::DumpDoc;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);

use Exporter;

@ISA = qw(Exporter);
@EXPORT_OK = qw( &mapUidGid );


use XML::DOM;
use XML::DOM::NodeList;
use MIME::Base64;
use File::Find;
use File::Copy;
use File::Basename;

use DBI;

use lib_module_db;
use Modules::Restore::Tables::Common;
use Modules::Restore::Tables::Common qw(&clean_tab($$));
use Modules::Restore::Map;
use Modules::Backup::Version;


use vars qw( $ptrMapUID $ptrMapGID $error);
$ptrMapUID = {};
$ptrMapGID = {};
$error='';


# $file = string. name of file containing the dump
# $obj_conf = object of Modules::Restore::DumpConfXML.
sub new($$) {
  my $class = shift;
  my ($file, $obj_conf) = @_;

  unless (-T $file && ref($obj_conf)=~/DumpConfXML/){
    printf STDERR "DumpDoc::new - erorr: wrong arguments\n";
    return undef;
  }

  my $this = {
	      'version_name' => '',
	      'version_code' => '',
	      'version_tag'  => '',
		
		# ref. to hash of handlers: 
		# 'table1' => { 'event1' => ref-to-routine, 'event2' => ref-to-routine }
		# called for each row of handled table, takes hash of fileds as an argument.
	      'tab_handlers'     => {},	

	      'file'		=> $file,
	      'obj_conf'	=> $obj_conf,
	      'attrs' => {'clean'=>0,'mode'=>'map','isBaseClean'=>undef},
	      'ip_replace' => undef,
	      'uid_replace'=>undef,
	      'map' => undef,
	     };
	# 
  return bless $this;
}

sub Map {
  my $this = shift;
  if (@_){
    my $map=shift;
    if(ref($map)=~/Map/){
      $this->{'map'}=$map;
    }
  }
  return $this->{'map'};
}

sub DumpConfXML{
  my $this=shift;
  if (@_){
    my $dumpConf = shift;
    if (ref($dumpConf)=~/DumpConfXML/){
      $this->{'obj_conf'}=$dumpConf;
    }
  }
  return $this->{'obj_conf'};
}


sub AttrMapping(){
  my $this=shift;
  if(@_){
    my $value=shift;
    $this->{'attrs'}->{'mode'}=$value?'map':'rest';
  }
  return ($this->{'attrs'}->{'mode'} eq 'map');
}

sub AttrRestore(){
  my $this=shift;
  if(@_){
    my $value=shift;
    $this->{'attrs'}->{'mode'}=$value?'rest':'map';
  }
  return ($this->{'attrs'}->{'mode'} eq 'rest');
}

sub Mode(){
  my $this=shift;
  if(@_){
    my $value=shift;
    if($value=~/rest/i){
      $this->{'attrs'}->{'mode'}='rest';
    }elsif($value=~/map/i){
      $this->{'attrs'}->{'mode'}='mapt';
    }
  }
  return $this->{'attrs'}->{'mode'};
}


sub AttrClean(){
  my $this=shift;
  if(@_){
    $this->{'attrs'}->{'clean'}=shift;
  }
  return $this->{'attrs'}->{'clean'};
}

sub Attribute(){
  my $this=shift;
  if(@_){
    my $name = shift;
    if (@_){
      $this->{'attrs'}->{$name}=shift;
    }
    return $this->{'attrs'}->{$name};
  }
  return $this->{'attrs'};
}


# set table handlers
# set table scpecific handlers 
# %handlers = hash of references to table handlers (ref-s to subroutines)
# sample: 
# 	$obj->setTableHandler(
#		'table1' => { 
#			'pre_table'  => \&handler1, 
#			'on_row'	 => \&handler2,
#			'post_table' => \&handler3
#		},
#		'*' => { 'on_row' => \&handler4 }
#		);

sub makeReplaceIP($){
  my $this = shift;
  my $map =shift;
  my $templ=shift || 'ip(*).address';
  unless (ref($map)=~/Map/){
    return 0;
  }
  $this->{'ip_replace'}=$map->getAttrHash($templ);
  my ($key,$val);
  foreach my $key (keys %{$this->{'ip_replace'}}){
    $val = $this->{'ip_replace'}->{$key};
    unless ($val=~/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/){
      delete $this->{'ip_replace'}->{$key};
    }
  }
}


sub replaceIP($){
  my $this = shift;
  my $srcIP = shift;
  if (ref($this->{'ip_replace'})=~/HASH/){
    if (exists($this->{'ip_replace'}->{$srcIP})){
      $srcIP=$this->{'ip_replace'}->{$srcIP};
    }
  }
  return $srcIP;
}

sub makeReplaceUID($){
  my $this = shift;
  my $map =shift || $this->{'map'};
  unless (ref($map)=~/Map/){
    return 0;
  }
  $map->initReplaceUID();
  $this->{'map'}=$map;
  return 1;
}

sub replaceUID($){
  my $this = shift;
  my $srcUID = shift;
  unless ($srcUID=~/\d+/){
    return undef;
  }
  if (ref($this->{'map'}=~/Map/)){
    $srcUID = $this->{'map'}->ReplaceUID($srcUID);
  }
  replace $srcUID;
}



sub setTableHandler(%) {
  my $this = shift;
  my (%handlers) = @_;
  foreach my $table (keys %handlers) {
    $$this{'tab_handlers'}{$table} = $handlers{$table};
  }
}


# proceed dump parsing and call the handlers specified
sub processDump {
  my $this = shift;
  my ($file, $obj_conf) = ($$this{'file'}, $$this{'obj_conf'});
	
  my $parser = new XML::DOM::Parser;
  printf STDERR "parsing file '$$this{'file'}'\n"; ## if $::DEBUG;
  my $xml_dump = $parser->parsefile($$this{'file'});
	
  my ($dump_main,$kid);
  foreach $kid ($xml_dump->getChildNodes){
    next unless ($kid->getNodeType==ELEMENT_NODE);
    if($kid->getNodeName eq 'confixx'){
      $dump_main=$kid;
      last;
    }
  }
  unless ($dump_main) {
    &errmsg("The XML dump is malformed: no 'confixx' node found\n");
    return 0; 
  }


	# gather information about version dumped
  $$this{'version_name'} = $dump_main->getAttribute('version_name');
  $$this{'version_code'} = $dump_main->getAttribute('version_code');
  $$this{'version_tag'}  = $dump_main->getAttribute('version_tag');
	
	# process dump and restore entities
	
  my $dumped_tables = $dump_main->getChildNodes();
  my $n_tables    = $dumped_tables->getLength();
	
  print STDERR "$n_tables to parse\n" if ($::DEBUG);
  warn("The dump is empty, no <tables> found.\n") if ($n_tables == 0);

  my ($ptrAttrs,$xml_table,$table_name,$obj_table_conf,$xml_rows,$xml_row,$i,$j,
      $n_rows,$table,$location,$path,$sqlLimit);
  # get configs
  # ...

  print STDERR "Pre phase\n";
  my (@tables,$ptrArr,$clean);
  $i=0;
  foreach $kid ($dump_main->getChildNodes){
    $i++;
    next unless ($kid->getNodeType==ELEMENT_NODE);
    next unless ($kid->getNodeName() eq 'table');
    unless($table_name = $kid->getAttribute('name')){
      &errmsg("Table-entry #$i of <confixx>-container has no 'name' attribute, ".
	      "so it will not be restored.\n");
      next;
    }
    $obj_table_conf = $obj_conf->getTable($table_name);			

    push @tables,[$table_name,$kid,$obj_table_conf];
  }
  
#
# pre phase
#  
  $clean=$this->AttrClean;
  foreach $ptrArr (@tables){
    ($table_name,$xml_table,$obj_table_conf) = @$ptrArr;

    print STDERR "Pre phase table: '$table_name'\n" if $::DEBUG;

    $this->callTableHandlers($table_name, 'pre_table',$obj_conf,$clean,$xml_table);
  }
#
# /pre phase
#


#
# clean phase
#
  if ($this->AttrRestore && $this->AttrClean){
    print STDERR "Clean phase\n";

    my $tblDelaccs = $obj_conf->getTable('delaccs');
    if(ref($tblDelaccs)=~/Table/){
      $tblDelaccs->AttrNoClean(1);
    }

    foreach $ptrArr (@tables){
      ($table_name,$xml_table,$table)=@$ptrArr;
      unless($table->StateClean ){
				print STDERR "Clean '$table_name'\n" if $::DEBUG;
				$this->callTableHandlers($table_name, 'clean',$obj_conf);
      }
    }

    print STDERR "launching confixx_updatescript.pl (clean phase)\n"; 
    my $cmd=$::installDir."/confixx_updatescript.pl --force-all".$main::skipCalc;
    $cmd.=' --debug' if ($::DEBUG);
    my $ret = system($cmd);
    if($ret){
      $ret>>=8;
      print STDERR "Can not clean Confixx: execution of '$cmd' failed:\n";
      if ($ret==13){
				print STDERR "Licence problem\n";
      }
    }
  }
#
# /clean
#
  if ($this->{'attrs'}->{'action'} eq 'clean'){
    print STDERR "Action: only clean. Confixx is clean\n" if $::DEBUG;
    return;
  }
#
# proccesing of rows

  print STDERR (($this->AttrRestore)?'Restoring ':'Mapping ')."of tables\n";

  my($xmlNode,$xmlRoot);
  foreach $ptrArr (@tables){
    ($table_name,$xml_table,$obj_table_conf)=@$ptrArr;
    if ($::DEBUG){
      print STDERR ($this->AttrRestore)?'Restore ':'Mapping ';
      print STDERR "table '$table_name'\n";
    }
    $sqlLimit=0;
    if($location=$xml_table->getAttribute('location')){
      $path=$this->{'file'};
      if($path=~s/\/[^\/]+$//){
				$path.="/$location";
      }else{
				$path=$location;
      }
      $xmlRoot = $parser->parsefile($path);
      foreach $kid ($xmlRoot->getChildNodes){
				next unless ($kid->getNodeType==ELEMENT_NODE);
				if($kid->getNodeName eq 'confixx'){
					$xmlNode=$kid;
					last;
				}
      }
#
# find table '$table_name'
#
      foreach $kid ($xmlNode->getChildNodes){
				next unless ($kid->getNodeType==ELEMENT_NODE);
				if($kid->getNodeName eq 'table'){
					if($kid ->getAttribute('name') eq $table_name){
						$xml_table=$kid;
						$sqlLimit=1;
						last;
					}
				}
      }
    }
    foreach $kid ($xml_table->getChildNodes()){
      next unless ($kid->getNodeType==ELEMENT_NODE);
      next unless ($kid->getNodeName() eq 'row');
      $ptrAttrs=$this->decode_row($kid);
      if (ref($ptrAttrs)=~/HASH/){
				$this->callTableHandlers($table_name, 'on_row', $ptrAttrs, $obj_table_conf);
      }
    }
    if($sqlLimit){
      $xmlRoot->dispose;
    }
  }
#
# / proccesing of rows
#

#
# post phase
#
  print STDERR "Post phase\n";
  foreach  $ptrArr (@tables){
    ($table_name,$xml_table,$obj_table_conf)=@$ptrArr;

    $this->callTableHandlers($table_name, 'post_table',$obj_conf,$clean,$xml_table);
  }
#
# /post phase
#

}


# call table handler
# $table_name = string. name of a table
# $event = string. event name
# @args = array of handler arguments.
sub callTableHandlers($$@) {
  my $this = shift;
  my ($table_name, $event, @args) = @_;
  my $ret;
	# call specific handler
  $ret=$this->callTableHandler($table_name, $table_name, $event, @args);
  if ($ret){
	# call common handler
    $ret= $this->callTableHandler('*'     , $table_name, $event, @args);
  }
  return $ret;
}

sub callTableHandler($$@) {
  my $this = shift;
  my ($table_name, $real_table_name, $event, @args) = @_;
  my $ret;
  unless (exists $$this{'tab_handlers'}{$table_name}) {
    return 1;
	}
  unless (defined $$this{'tab_handlers'}{$table_name}{$event}) {
    return 1;
  }
  unless (ref($$this{'tab_handlers'}{$table_name}{$event}) eq 'CODE') {
    &errmsg("handler of $event($table_name) is not valid");
    return 0;
  }
  $ret = & {$$this{'tab_handlers'}{$table_name}{$event}} ($real_table_name, @args);
  return $ret;
}

# $obj_attributes = object of XML::DOM::NodeAttributesMap. attributes of table row.
# $obj_table_conf = object of Modules::Restore::Table. settings of the table.
sub decode_attrs($$) {
  my $this = shift;
  my ($obj_attributes, $obj_table_conf) = @_;
  my %attrs = ();
  for (my ($i, $n_attrs) = (0, $obj_attributes->getLength()); $i < $n_attrs; $i++) {
    my $attr_name  = $obj_attributes->item($i)->getName();
    my $attr_value = $obj_attributes->item($i)->getValue();
    $attrs{$attr_name} = ($obj_table_conf->isPropSafe($attr_name))
			? $attr_value
			: decode_base64($attr_value);
  }
  return %attrs;
}

sub decode_row($){
  my $this = shift;
  my $xmlNode = shift;
  my %attrs;
  unless(ref($xmlNode)=~/XML/){
    return undef;
  }

  my ($xmlField,$fldName,$fldValue,$fldEncode,$i,$nodeName);

  foreach $xmlField ($xmlNode->getChildNodes()){
    next unless ($xmlField->getNodeType==ELEMENT_NODE);
    $i++;
    $nodeName = $xmlField->getNodeName();
    next unless ( $nodeName =~ /^field$/i);

    unless ($fldName = $xmlField->getAttribute('name')){
      &errmsg("Field-entry #$i has no 'name' attribute, so it will not be restored.\n");
      next
    }

    next if ($xmlField->getAttribute('null'));

    $fldValue= $xmlField->getAttribute('value');
    unless (defined($fldValue)){
      $fldValue=($xmlField->getAttribute('type')=~/I/i)?0:'';
    }
    $fldEncode= $xmlField->getAttribute('encode');
    if ($fldEncode=~/base64/i && $fldValue){
      $attrs{$fldName}=decode_base64($fldValue);
    }else{
      $attrs{$fldName}=$fldValue;
    }
  }
  return \%attrs;
}

sub errmsg {
  my ($arg_error) = @_;
  $error = $arg_error;
}

sub restoreFiles {
  my $this = shift;
  my ($dumpDir,$newOwnership) = @_;

  return 0 unless (-d $dumpDir);
  my $dumpConf = $this->{'obj_conf'};

  return 0 unless ( ref($dumpConf) =~ /DumpConfXML/ );
  my $map = $this->{'map'};

  unless ( ref($map) =~ /Map/ ){
    $map = $::map;
    $this->{'map'}=$map;
  }

	$::bin_tar ||= $::bin{'tar'} || 'tar';
	$::bin_cat ||= $::bin{'cat'} || 'cat';
	$::bin_cp ||= $::bin{'cp'} || 'cp';

  my ($secName,$srcDir,$dstDir,$each,@tgzs,$srcTgz,$eachTmpl,
      $newEach,$cmd,$ptrBack,$cmdAppend,$appendDir,$dir,$file,
      $fileType,$fileKey,$fileSep,@content,$line,@filds,%keys,
      $key,$srcFile,$dstFile,$fileMode,$fileGid,$fileUid,$srvName,
      $service,$restoreRule,$ok,@forDelete,%prepEach,%dirs,
      $chunk,$path,$srcEachDir,$dstEachDir,$ptrEachMap,$tblName,$fldName);
  my ($minLevel,$level,$table,$section,$base,$val,$excl,$cnt,%srcTgzs);

  $Modules::Restore::DumpDoc::ptrMapUID = $map->getAttrHash( 'user(*).uid' );
  $Modules::Restore::DumpDoc::ptrMapGID = $map->getAttrHash( 'group(*).gid' );

	if ( $newOwnership ) { ## user's GID is same as UID
		my ($oldUid,$newUid);
		while (($oldUid,$newUid) = each(%{$Modules::Restore::DumpDoc::ptrMapUID})){
			unless ( $oldUid eq $newUid || 
							 exists $Modules::Restore::DumpDoc::ptrMapGID->{$oldUid}) {
				$Modules::Restore::DumpDoc::ptrMapGID->{$oldUid} = $newUid;
			}
		}
	}

  foreach $secName ( $dumpConf->getSectionsList() ) {
    $section = $dumpConf->getSection( $secName );

#check type of object
    next unless ( ref($section)=~/Section/ ); ## check type

    unless( $section->checkIf() ) {
      print STDERR "Value of 'if'-attribute is false. Section '$secName' is skiped\n" if $::DEBUG;
      next;
    }

    if ( ( $srvName = $section->Attribute( 'service' ) ) &&
       ( $service = $dumpConf->getService( $srvName ) ) ) {

      if ( $service->isStateOff() ) {
	 ## skip if service is off
				print STDERR "Corresponded service '$srvName' is of. ".
					"Section '$secName' is skipped\n" if $::DEBUG;
				next;
      }
    }

    $restoreRule = $::map->get_rule($section->Attribute('rule'))
      || $Modules::Restore::Map::APPEND_DUMPED;

    $srcDir = "$dumpDir/$secName";

    print STDERR "Section: $secName ($srcDir)\n" if $::DEBUG;

    next unless (-d $srcDir);

    $base = $section->Base();

    unless ( $base = $section->getPrepared( $base ) ) {
			warn "Can not prepare '$base'\n" if $::DEBUG;
      next;
    }

    $eachTmpl = $section->Each();

    if ( $eachTmpl eq "#user.name#" ) {
      $ptrEachMap = $map->getAttrHash( 'user(*).name' );

    } elsif ( $eachTmpl eq "#domain.id#" ) {
      $ptrEachMap = $map->getAttrHash( 'domain(*).id' );

    } elsif ( $eachTmpl=~/#(\S+)\.(\S+)#/ ) {
      $tblName = $1;
      $fldName = $2;
      $key = $tblName.'(*).'.$fldName;
      $ptrEachMap = $map->getAttrHash($key);
      unless ( keys %{$ptrEachMap} ) {
				if ( $table = $dumpConf->getTable( $tblName ) ) {
					$ptrEachMap = $table->getMap( $fldName );
				}
      }
      %prepEach=();
      while ( ($key,$val) = each( %{$ptrEachMap} ) ) {

				$key =~ s/^\///;
				$val =~ s/^\///;
				$prepEach{$key}=$val;
      }
      $ptrEachMap = \%prepEach;
    }else{
      $ptrEachMap = {};
    }

#
# get list of tars
#
    @tgzs = ();
    find sub {  -f && /\.tgz(?:\.\w+)?$/ && push @tgzs,$File::Find::name }, $srcDir;

		%srcTgzs=();
    foreach $srcTgz ( @tgzs ) {
			if ( $srcTgz =~ /(.*\.tgz)\.\w+$/ ) {
				$srcTgz = $1;
				$srcTgzs{$srcTgz} = $srcTgz.'.*' unless exists $srcTgzs{$srcTgz};
			} else {
				$srcTgzs{$srcTgz} = 0;
			}
		}

		my $reDir = qr/^\Q$srcDir\E\//;

    foreach $srcTgz (keys %srcTgzs){

      $each = $srcTgz;
      $each =~ s/$reDir//; ## remove prefix
      $each =~ s/\.tgz$//; ## remove extension

			if ( $srcTgzs{$srcTgz} ) {
				$cmd = $::bin_cat.' '.$srcTgzs{$srcTgz}.
					"|$::bin_tar -p --numeric-owner -x -z ";
			} else {
				$cmd = "$::bin_tar -p --numeric-owner -x -z -f $srcTgz "; ## restore mode & numeric uid/gid
				next unless(-f $srcTgz);
			}

      print STDERR "Source: $srcTgz\n" if $::DEBUG > 1;

      if ( $eachTmpl ) {
				unless ( $newEach = $ptrEachMap->{$each} ) {
					print STDERR "Not found mapping for '$each'. File '$srcTgz' is skiped\n" if $::DEBUG;
					next;
				}
      }

      $dstDir = $eachTmpl? $section->prepareEach( $base,$newEach ): $base;
      unless(-d $dstDir){
				print STDERR "Restore files error: dir '$dstDir' not found\n";
				next;
      }

#
# delete for restore
#
      if ( $section->Attribute( 'delete' ) ) {
				my ($dir,$del,$delPath,$realPath);
				foreach $dir ( @{$section->Dirs()} ) {
					next unless ref($dir) =~ /Dir/;
					$dir->Base($dstDir);
					
					foreach $del ( @{$dir->Delete} ) {
						next unless ref($del) =~ /Path/;
						$delPath = $del->Path();
						$realPath = $dir->fullPath( $delPath );
						if ( -f $realPath ) {
							unlink $realPath;
						}
					}
				}
      }
#
# end delete for restore
#

#
#     always use temporary dir to chown
#
			$appendDir="$srcDir/$each.temp";
			if (-d $appendDir){
				system( "rm -rf $appendDir" );
			}
			mkdir $appendDir,0750;

			$cmdAppend = "$cmd -C $appendDir";
			$cmdAppend .= " 1>/dev/null 2>&1" unless ($::DEBUG);
			print STDERR "Run command: $cmdAppend\n" if $::DEBUG>1;
			unless ( system( $cmdAppend ) == 0 ) {
				warn "Unable restore files's content from $srcTgz to $appendDir: $!\n";
				next;
			}

      if ( $section->Attribute( 'append' ) ) {

				foreach $file ( @{$section->Files()} ) {

					$fileType = $section->FileAttribute( $file,'type' );
					next unless( $fileType =~ /append/ );

					$fileKey = $section->FileAttribute( $file,'key' );
					$fileSep = $section->FileAttribute( $file,'separator') || ':';

					if ( $eachTmpl ) {
						$srcFile=$appendDir.'/'.$section->prepareEach( $file,$each );
						$dstFile=$dstDir.'/'.$section->prepareEach( $file,$newEach );
					} else {
						$dstFile = "$dstDir/$file";
						$srcFile = "$appendDir/$file";
					}

					($fileMode,$fileUid,$fileGid)=(stat($srcFile))[2,4,5];
					$fileMode &= 07777;
					if ( $restoreRule == $Modules::Restore::Map::ASSIGN_CURRENT ) {
						
						copy ($dstFile,$srcFile); ## save current file

					}else{
						if ( $fileKey && ( $fileKey=~/\d+/ ) ) {
							open(IN,"< $srcFile") or next;

							@content=(<IN>);
							close(IN);
							%keys=();

							open(IN,"< $dstFile") or next;
							open(OUT,"> $srcFile") or next;

							$fileKey--;
							while ( <IN> ) {
								$line = $_;
								$key = (split($fileSep,$line))[$fileKey];
								$keys{$key} = 1;
								print OUT $line;
							}
							close (IN);

							foreach $line (@content){
								$key = (split($fileSep,$line))[$fileKey];
								next if (exists ($keys{$key}));
								print OUT $line;
							}
							close (OUT);
						}else{
							$cmdAppend = "cat $srcFile >>$dstFile";
							unless( system ( $cmdAppend ) == 0 ) {
								warn "Unable run command $appendDir: $!\n";
								next;
							}
							unlink $srcFile;
							copy ($dstFile,$srcFile); ## save temp file
						}
					}
					chown $fileUid,$fileGid,$srcFile;
					chmod $fileMode,$srcFile;
				}
      }

      %dirs = ();
      if ($section->orAttributes('each','exclude')){
#
# mapping file names
#
				foreach $file (@{$section->Files}){
					$srcFile = $section->prepareEach( $file,$each );
					$dstFile = $section->prepareEach( $file,$newEach );
					if ( ($restoreRule == $Modules::Restore::Map::ASSIGN_CURRENT ) &&
							( -f "$dstDir/$dstFile" ) ) {
						unlink "$appendDir/$srcFile";
						next;
					}
					
					@filds=split(/\//,$dstFile);
					pop @filds;
					&accumDirs(\@filds,\%dirs);
					
					next if ($srcFile eq $dstFile);
					rename "$appendDir/$srcFile","$appendDir/$dstFile";
				}

				foreach $dir ( @{$section->dirNames()} ) {

					$srcEachDir = $section->prepareEach($dir,$each);
					$dstEachDir = $section->prepareEach($dir,$newEach);

					if ( ($restoreRule == $Modules::Restore::Map::ASSIGN_CURRENT ) &&
							( -d "$dstDir/$dir" ) ) {
						
						system ("rm -fR $appendDir/$srcEachDir");
						next;
					}
					@filds=split(/\//,$dstEachDir);
					&accumDirs(\@filds,\%dirs);

					next if ($srcEachDir eq $dstEachDir);
					rename "$appendDir/$srcEachDir","$appendDir/$dstEachDir";
				}
      }

      if ( $section->Attribute('exclude' ) ) {

				my ($excl,$exclPath,$table,$field,$exclWhere,$regExp,$wherePath,
						$realPath,$sql,$sth,$tblExcl);

				foreach $dir ( @{$section->Dirs()} ) {
					next unless ref($dir) =~ /Dir/;
					$dir->Base($appendDir);

					foreach $excl ( @{$dir->Exclusions()} ) {
						next unless ref($excl) =~ /Path/;

						$exclPath=$excl->Path();

						$regExp = qr/#([^.]*)\.([^.]+)#/; ## compile regular expression
						if ( $exclPath =~ /$regExp/ ) {
							$table = $1;
							$field = $2;
							next unless ($table && $field);
#
# check table & field
#
							$tblExcl = $dumpConf->getTable($table); ## there is table
							unless ( ref($tblExcl) =~ /Table/ ) {
								print STDERR "DumpDoc::restoreFiels: table '$table' is not found.".
									" Exclude '$exclPath' is skipped\n" if $::DEBUG;
								next;
							}
							unless (exists($tblExcl->getFields()->{$field})){
								print STDERR "DumpDoc::restoreFiels: table '$table' has not field '$field'.".
									" Exclude '$exclPath' is skipped\n" if $::DEBUG;
								next;
							} ## there is field
#
# end check table & field
#
							$sql="select $field from $table";
#
# where-attribute
#
							if ( $exclWhere = $excl->Where() ) {
								$exclWhere =~ s/\$each\$/$newEach/g;
								if ( $exclWhere =~ /$regExp/){
									$field = $2;
									if ( $1 eq $table) {
										if ( exists( $tblExcl->getFields()->{$field}) ) {
											$exclWhere =~ s/$regExp/$field/;
											$sql .= " where $exclWhere";
										}else{
											print STDERR "DumpDoc::restoreFiels: table '$table' has not field '$field'.".
												" 'where'- attribute of exclude '$exclPath' is skipped\n" if $::DEBUG;
										}
									}else{
										print STDERR "DumpDoc::restoreFiels: tables of exclude '$exclPath' are different.".
											" 'where'-attribute of exclude '$exclPath' is skipped\n" if $::DEBUG;
									}
								}
							}
#
# end where-attribute
#

							$sth = $::dbh->prepare( $sql );
							if ( $sth && $sth->execute() ) {
								while ( ($wherePath) = $sth->fetchrow() ) {
									($realPath = $exclPath) =~ s/$regExp/$wherePath/;
									$realPath=$dir->fullPath($realPath);
									if ( -f $realPath ) {
										unlink $realPath;
									} elsif ( -d $realPath ) {
										system ("rm -fR $realPath");
									}
								}
								$sth->finish;
							}else{
								print STDERR "DumpDoc::restoreFiels: error exequte sql: '$sql'\n".
									"$DBI::errstr\n" if $::DEBUG;
							}
						}
					}
				}
      }


      $minLevel = &prepareDirs(\%dirs);
      if(-d $appendDir){
				if ((!$section->orAttributes('append','each','exclude')) &&
						($restoreRule == $Modules::Restore::Map::ASSIGN_CURRENT) &&
						(-d "$dstDir")
					 ){
					$cmd =''; ## skip
				}else{

					if($minLevel){

						$ok=1;
						while (($path,$level)=each(%dirs)){

#
# map uid/gid
#	
							find ( \&Modules::Restore::DumpDoc::mapUidGid, "$appendDir/$path" );

#
# check dir's tree
#
							&checkDir("$dstDir/$path","$appendDir/$path");

#
# check if not empty
#
							$cnt=0;
							if (opendir (SRC,"$appendDir/$path")){
								while ($srcFile=readdir(SRC)){
									unless ($srcFile=~/^\.\.?$/){
										$cnt++;
										last;
									}
								}
								closedir (SRC);
							}
							
							if($cnt){
#		$cmd="$::bin_cp $appendDir/$path/* $dstDir/$path -dfpR";
#		print STDERR "Run command: $cmd\n" if $::DEBUG>1;
#		unless (system($cmd)==0){
#		  $ok=0;
#		  print STDERR "Unable run command: $cmd\n";
#		}
								if(opendir (SRC,"$appendDir/$path")){
									my $src;
									while ($src=readdir(SRC)){
										if($src=~/^\.\.?$/){
											next;
										}

										$src=~s/(\'|\"|\$|\s|\\|\~)/\\$1/g;  ## shild special symbols

										$cmd="$::bin_cp $appendDir/$path/$src $dstDir/$path -dfpR";
										print STDERR "Run command: $cmd\n" if $::DEBUG>1;
										unless (system($cmd)==0){
											$ok=0;
											print STDERR "Unable run command: $cmd\n";
										}
									}
									closedir(SRC);
								}
							}else{
								print STDERR "Warning: dir '$appendDir/$path' is empty\n" if $::DEBUG>1;
							}
							$cmd='';
						}
					}else{
						$cmd='';
						if(opendir (SRC,$appendDir)){
#
# check if not empty
#
							while ($srcFile=readdir(SRC)){
								unless ( $srcFile =~ /^\.\.?$/ ) {
									$cmd = "$::bin_cp $appendDir/* $dstDir -dfpR";
									last;
								}
							}
							closedir(SRC);
						}
						if ( $cmd ) {
#
# map uid/gid
#
							find ( \&Modules::Restore::DumpDoc::mapUidGid, $appendDir );

#
# check dir's tree
#
							&checkDir($dstDir,$appendDir);
						}
					}
				}
      }
	
      if ( $cmd ) {
				print STDERR "Run command: $cmd\n" if $::DEBUG>1;
				if ( system( $cmd ) == 0 ) {
					$ok = 1;
				}else{
					$ok = 0;
					warn "Unable restore files's content from $srcTgz to $dstDir: $!\n";
					next;
				}
      }

      if (-d $appendDir ) {
#
# clean temporary dir
#
	system("rm -rf $appendDir");
      }
    }
  }
}

sub checkDir {
  my ($dstDir,$srcDir)=@_;
  return 1 if (-d $dstDir);

  if ( ( $dstDir =~ /^\// ) xor ( $srcDir =~ /^\// ) ) {
    print STDERR "Error: DumpDoc::checkDir: Malformed arguments: '$dstDir','$srcDir'\n";
    return 0;
  }

  my @dstChunks = split(/\//,$dstDir);
  my @srcChunks = split(/\//,$srcDir);

  my (@dstTail,@srcTail,$dstChunk,$srcChunk,$dstPath,$srcPath);

  while (@dstChunks){
    $dstChunk = pop @dstChunks;
    push @dstTail,$dstChunk;

    $srcChunk = pop @srcChunks;
    push @srcTail,$srcChunk;

    $dstPath = join('/',@dstChunks);
    last if (-d $dstPath);
  }

  return 0 unless (-d $dstPath);

  my ($mode,$uid,$gid);
  $srcPath= join('/',@srcChunks);

  while (@dstTail){
    $dstChunk = pop @dstTail;
    $dstPath.='/'.$dstChunk;
    $srcChunk = pop @srcTail;
    $srcPath.='/'.$srcChunk;

    if (-d $srcPath){
      ($mode,$uid,$gid)=(stat($srcPath))[2,4,5];
      $mode &= 07777;
    }
    if ( $::DEBUG ) {
      my $params=sprintf('mode: %04o, uid: %u, gid; %u',$mode,$uid,$gid);
      print STDERR "checkDir: create dir '$dstPath' ($params)\n";
    }
    if ($mode){
      mkdir $dstPath,$mode or
				warn "Error: unable ot make dir '$dstPath': $!\n";
    }else{
      mkdir $dstPath or
				warn "Error: unable ot make dir '$dstPath': $!\n";
    }
    if ($uid && $gid){
      chown $uid,$gid,$dstPath or
				warn "Error: unable to chown '$dstPath': $!\n";
    }
  }
  return 1;
}

sub prepareDirs {
  my $ptrDirs=shift;
  my ($minLevel,$path,$level);
  if ( !(values %{$ptrDirs}) ||
       exists ($ptrDirs->{'/'})
     ){
    $minLevel=0;
  }else{
    $minLevel=1000;
    foreach $level (values %{$ptrDirs}){
      $minLevel=$level if $level<$minLevel;
    }
    foreach $path (keys %{$ptrDirs}){
      if ($ptrDirs->{$path}>$minLevel){
	delete $ptrDirs->{$path};
      }
    }
  }
  return $minLevel;
}

sub accumDirs {
  my ($ptrChunks,$ptrDirs)=@_;
  my ($path,$level,$chunk);
  unless (exists($ptrDirs->{'/'})){
    $path='';
    $level=0;
    foreach $chunk (@{$ptrChunks}){
      $path.='/' if $path;
      $path.=$chunk;
      $level++;
      last if exists($ptrDirs->{$path});
    }
    $ptrDirs->{$path}=$level;
  }
}

sub mapUidGid{

  my($uid,$gid)=(stat($File::Find::name))[4,5] or
		return;

  my ($go,$newValue );

	if ( exists( $Modules::Restore::DumpDoc::ptrMapUID->{$uid} ) ) {
		$newValue = $Modules::Restore::DumpDoc::ptrMapUID->{$uid};
		if ( ( $newValue =~ /^\d+$/ ) && ($uid != $newValue ) ) {
			$uid = $newValue;
			$go = 1;
		}
  }

	if ( exists($Modules::Restore::DumpDoc::ptrMapGID->{$gid} ) ) {
		$newValue=$Modules::Restore::DumpDoc::ptrMapGID->{$gid};
		if ( ( $newValue =~ /^\d+$/ ) && ( $gid != $newValue ) ) {
			$gid = $newValue;
			$go = 1;
		}
  }

  if ( $go ) {
    chown ($uid,$gid,$File::Find::name) or
			warn "Error: unable to chown '$File::Find::name': $!\n";
  }
}

sub restoreMySQL ($) {
  my $this = shift;
  my $dumpDir = shift;
  return 0 unless (-d $dumpDir);

  return 1 unless( $::mysqlUserServer && $::mysqlUserUser);

  my $dumpConf = $this->{'obj_conf'};
  return 0 unless (ref($dumpConf)=~/DumpConfXML/);
  my $map = $this->{'map'};
  unless (ref($map)=~/Map/){
    $map = $::map;
    $this->{'map'} = $map;
  }

  my $tblMySQL=$dumpConf->getTable('mysql_datenbanken');
  return 0 unless (ref($tblMySQL)=~/Table/);

  opendir (DIR,$dumpDir);
  my @tgzs = grep{ /.+\.tgz(?:\.\w+)?$/ && (-f "$dumpDir/$_") } readdir(DIR);
  closedir(DIR);

  return 1 unless @tgzs; ## nothing

	my (%srcTgzs,$srcTgz);
	foreach $srcTgz ( @tgzs ) {
		if ( $srcTgz =~ /(.*\.tgz)\.\w+$/ ) {
			$srcTgz = $1;
			$srcTgzs{$srcTgz} = $srcTgz.'.*' unless exists $srcTgzs{$srcTgz};
		} else {
			$srcTgzs{$srcTgz} = 0;
		}
	}

	$::bin_tar ||= $::bin{'tar'} || 'tar';
	$::bin_cat ||= $::bin{'cat'} || 'cat';
	$::bin_su ||= $::bin{'su'} || 'su';
	$::bin_mysql ||= $::bin{'mysql'} || 'mysql';


  my $rdb_address = &makeDSN('mysql',$::mysqlUserServer,'mysql',$::mysqlUserPort || $::mysql_sock);

  my $rdbh;
  unless( $rdbh = DBI->connect($rdb_address, $::mysqlUserUser, $::mysqlUserPw ) ) {
    print STDERR "Modules::Restore::DumpDoc::restoreMySQL : Error connect to MySQL: $DBI::errstr\n";
    return 0;
  }

  my ( $tgz, $cmd, $oldUser, $newUser, $key, $userDir, @sqls,
			 $oldDbName, $newDbName, $sql, $line, $dbName, $suffix, $go, $rsth,
			 $crypt, $userPwd, $userHost, $pw, $whereUser, $criptPasswd, $slash );

  my $restorePassword = ($map->get_rule("mysql(*).Password")==$RESTORE_DUMPED)?1:0;
	my $host = $::mysqlUserHost || $::mysql_host || 'localhost';

  foreach $tgz (keys %srcTgzs){
#
# user's loop
#
    next unless ($tgz =~ /^(.+)\.tgz(?:\.\w+)?$/);
    $oldUser = $1;
    $key = "user($oldUser).name";
    $newUser = $map->get_attr($key)||$oldUser;

#    $userDir="$dumpDir/$newUser";
    $userDir = $::user_homeDir."/$newUser/restore";
    unless (-d $userDir){
			print STDERR "DumpDoc::restoreMysql : Error. User home dir is nod found: $userDir\n";
			next;
    }
    $userDir .= "/mysql";

    if ( -d $userDir ) {
      system("rm -rf $dumpDir");
    }
    unless (mkdir ($userDir,0755)){
      print STDERR "Modules::Restore::DumpDoc::restoreMySQL : Error. Can not create dir '$userDir': $!\n";
      return 0;
    }

    $cmd = "$::bin_tar -xz -C $userDir";

		if ( $srcTgzs{$tgz} ) {
			$cmd = "$::bin_cat $dumpDir/".$srcTgzs{$tgz}.'|'.$cmd;
		} else {
			$cmd .= " -f $dumpDir/$tgz";
		}

    $cmd .= " 1>/dev/null 2>&1" unless ($::DEBUG);
    print STDERR "Run command: $cmd\n" if $::DEBUG;

    unless ( system( $cmd ) == 0 ) {
      warn "Unable restore MySQL's content from $dumpDir/$tgz to $userDir: $!\n";
      next;
    } 
    opendir (DIR,$userDir);
    @sqls = grep{!/^\./ && (-T "$userDir/$_")} readdir(DIR);
    closedir(DIR);

    next unless @sqls; ## empty

    if ( $restorePassword ) {
      $criptPasswd = $map->get_attr("mysql($oldUser).Password");
    }else{
      $criptPasswd = '';
    }
    $whereUser = "(user='$newUser')";

    $sql="SELECT password,host FROM user WHERE $whereUser ".
			" AND host IN ('$host','%','127.0.0.1','localhost')";
    $rsth=$rdbh->prepare($sql);
    $rsth->execute;

    unless ($rsth->rows){
      
      print STDERR "Modules::Restore::DumpDoc::restoreMySQL : ".
				"Error. User '$newUser' not found on $::mysqlUserServer\n";
      next;
    }

    ( $userPwd, $userHost ) = $rsth->fetchrow;  # save user password
		if ( $restorePassword ) {
			$criptPasswd ||= $userPwd; ## try to restore user's pasword
		}

    $rsth->finish;

#
# generate new password
    $key = "user($newUser).mysqlpwd";
    unless ( $pw = $map->get_attr( $key ) ) {
      $pw = join ( '',  map { (0..9, 'A'..'Z', 'a'..'z')[ rand( 64 ) ] } ( 1..8 ) );
    }

    unless ( $criptPasswd ) {
      $map->add_attr( $key, $pw ); # register changes
    }
#
# generate random password

    $rdbh->do( "UPDATE user SET password=PASSWORD('$pw') WHERE $whereUser" );
    $rdbh->do( "FLUSH PRIVILEGES" );         # set new password


    foreach $oldDbName ( @sqls ) {
#
# database's loop
#
      $newDbName = $tblMySQL->getMappedValue('dbname',$oldDbName) || $oldDbName;
      if ( $newDbName eq $oldDbName ) {

				$sql = "$userDir/$oldDbName";

      } else {
#
# copy dump with replacing $oldDbName => $newDbName
#
				$sql = "$userDir/$newDbName.sql";

				open(IN,"< $userDir/$oldDbName") or next;
				unless ( open( OUT, '>', $sql ) ) {
					print STDERR "Modules::Restore::DumpDoc::restoreMySQL : Error. Can not create file '$sql': $!\n";
					next;
				}
				$go = 0;
				while( <IN> ) {
					if ( $go ) {
						print OUT;
						next;
					}
					$line = $_;
					if ( $line =~ /^\s*use\s+([`']?)([\w\d_]+)\1;(.*)$/i ) {
						$slash = $1;
						$dbName = $2;
						$suffix = $3;
						if ( $dbName eq $oldDbName ) {
							$line = "USE $slash$newDbName$slash;$suffix\n";
							$go = 1;
						}
					}
					print OUT $line;
				}
				close (OUT);
				close (IN);
				unlink "$userDir/$oldDbName";
	
      }

#
# clean database
#
      $rdbh->do("DROP DATABASE $newDbName");
      $rdbh->do("CREATE DATABASE $newDbName");  
      $rdbh->do("FLUSH PRIVILEGES");

			chmod (0664,$sql); ## read for all	

#
# calculate command line
#	
      $cmd = "$::bin_cat $sql | $::bin_mysql -f ". &makeCmdLine( { 'user' => $newUser,
																																	 'password' => $pw,
																																	 'host' => $::mysqlUserServer,
																																	 'port' => $::mysqlUserPort }
																															 );
      $cmd .= " 1>/dev/null 2>&1" unless ($::DEBUG);

      if ((-x $::bin_su) && (-x $::stdShell)){
				$cmd = "$::bin_su - $newUser -s $::stdShell -c '$cmd'";
      }

      print STDERR "Run command: $cmd\n" if $::DEBUG;
      if (system($cmd)){
				print STDERR "Error execute: $cmd\n";
      }
#
# clean
#
      unlink $sql;
#
# end database's loop
#
    }
    system ("rm -rf $userDir");

#
# restore  password
#
    if ( $criptPasswd ) {
      $rdbh->do( "UPDATE user SET Password='$criptPasswd' WHERE $whereUser");
      $rdbh->do( "FLUSH PRIVILEGES");
      $map->del_attr($key); # unregister password

			print STDERR "User's mysql-password is restored from dump\n" if $::DEBUG;
		}
#
# end user's loop
#
  }

  $rdbh->disconnect;
}

sub getVersion {
	my $this = shift;

	my $version = Modules::Backup::Version->new( $this->{'version_code'} );

	return $version;
}

1;
