package Modules::Restore::Map;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

use XML::DOM;

BEGIN {
    use Exporter;
    @ISA = qw(Exporter);
    @EXPORT = qw($ASSIGN_CURRENT $RESTORE_DUMPED $APPEND_DUMPED &get_rule_name
		&defaultRule &initReplaceUID &LastUID &ReplaceUID
		&parseSystem &add_rule &get_rule &del_rule &add_attr &get_attr $del_attr); 
}

use strict;
no strict "vars";
no strict "refs";

*ASSIGN_CURRENT = \1;
*RESTORE_DUMPED = \2;
*APPEND_DUMPED = \3;

sub new {
#	my ($file_name) = @_;
  my $this = {
	      'rules' => {},
	      'attrs' => {},
	      'oldMinUID'=>0,
	      'oldUserPrefix'=>'web',
	      'newMinUID'=>$main::min_uid,
	      'newMaxUID'=>$main::max_uid,
	      'lastUID'=>$main::min_uid,
	      'last0UID'=>$main::min_uid,
	      'lastUIDisUnChecked'=>1,
	      'oldMainConfig'=>'',
	      'checkLastUID'=>1,
	     };
  return bless($this);
}

sub parseOldConfig($) {
  my $this = shift;
  my $confpath = shift;
  my %maps = ( 'group.name' =>
							 { 'userGroup'=> undef,
								 'popGroup'=> undef,
								 'apachegroup'=> undef,
								 'confixx_group'=> undef
							 },
							 'group.gid' => 
							 { 'giduser'=> undef,
								 'gidpop'=> undef,
								 'gidapache'=> undef,
								 'confixx_gid'=> undef,
								 'fp_gid'=> undef, 
								 'mta_gid'=> undef
							 },
							 'user.name' => 
							 { 'majordomo_user'=> undef
							 },
							 'user.uid' => 
							 { 'confixx_uid'=> undef,
								 'mta_uid'=> undef
							 },
						 );

  
  $confpath = $this->oldMainConfig($confpath);
  if ( -T $confpath ) {
    my ($key,$value,$mapName,$map,$newValue,$ptrMap);
    my ($minUID,$state,$userPrefix,$prefix,$suffix,$mainKey);
    open (CONF,"<$confpath");
    $state = 2;
    while(<CONF>){
      unless  ( /^\s*\$(\S+)\s*=\s*(\S+)/){
				next;
      }
      $key = $1;
      $value = &extractValue($2);
      unless($value){
				next;
      }
      if ( $key eq 'min_uid' ) {
				$this->{'oldMinUID'} = $value;
      } elsif ( $key eq 'user_prefix'){
				$this->{'oldUserPrefix'} = $value;
      } else {
				while ( ($mapName,$map) = each %maps ) {   # loop by maps
					if ( exists( $map->{$key} ) ) {         # check key in map
						$mainKey = "main::$key";
						$newValue = $$mainKey;               # get new value
						$map->{$key} = [$value, $newValue]; # store mapping $value -> $newValue
						last;                            # found. go to next line of _main.conf
					}
				}
      }
    }
    close(CONF);
    while( ($mapName,$map) = each %maps ) {   # loop by maps
      ($prefix,$suffix) = split(/\./,$mapName);
      foreach $ptrMap (values %{$map}){
				unless (ref($ptrMap)=~/ARRAY/){
					next;
				}
				if($ptrMap->[0] && $ptrMap->[1]){
					$key = $prefix.'('.$ptrMap->[0].').'.$suffix;
					$this->add_attr($key,$ptrMap->[1]);
				}
      }
    }
    $this->{'oldMainConfig'}=$confpath;
  }
  return $confpath;
}
sub extractValue ($){
  my $val = shift;
  $val =~ s/(.+);$/$1/;
  $val =~ s/^\'([^\']+)\'$/$1/;
  $val =~ s/^\"([^\"]+)\"$/$1/;
  return $val;
}

sub oldMainConfig {
  my $this = shift;
  if ( @_ ) {
    my $path = shift;
    if ( -T $path ) {
      $this->{'oldMainConfig'} = $path;
    }
  }
  return $this->{'oldMainConfig'};
}

sub oldUserPrefix {
  my $this = shift;
  return $this->{'oldUserPrefix'};
}

sub initReplaceUID {
  my $this = shift;
	my ($compl);
  ($this->{'last0UID'},$compl) = &getMaxUID("kunde='".$::user_prefix."0'");
	$this->{'complicit'} ||= $compl;

  $this->{'lastUID'} = &getMaxUID();
  return $this->{'lastUID'};
}

sub getMaxUID {
  my  $where=shift;
  my ($maxPopUID,$maxFtpUID,$maxUID,$sql,$sth,$coml);

  $maxUID = $main::min_uid;

  $sql = "select max(uid) from kunden";
  if ( $where ) {
    $sql .=" where $where";
  }

  $sth = $main::dbh->prepare($sql);
  if ( $sth->execute ) {
    if ( $sth->rows ) {
      ($maxUID) = $sth->fetchrow;
      if ( $maxUID < $main::min_uid ) {
				$maxUID = $main::min_uid;
      }
    }
    $sth->finish;
  }

  $sql = "select max(uid) from pop3";
  if ( $where ) {
    $sql .= " where $where";
  }

  $sth = $main::dbh->prepare($sql);
  if ( $sth->execute ) {
    if ( $sth->rows ) {
      ($maxPopUID) = $sth->fetchrow;
			$compl ||= $maxPopUID;
      if ( $maxUID < $maxPopUID ) {
				$maxUID = $maxPopUID;
      }
    }
    $sth->finish;
  }

  $sql = "select max(uid) from ftp";
  if ( $where ) {
    $sql .= " where $where";
  }

  $sth = $main::dbh->prepare($sql);
  if ( $sth->execute ) {
    if ( $sth->rows ) {
      ($maxFtpUID) = $sth->fetchrow;
			$compl ||= $maxFtpUID;
      if ( $maxUID<$maxFtpUID ) {
				$maxUID = $maxFtpUID;
      }
    }
    $sth->finish;
  }
	if ( wantarray() ){
		return ($maxUID,$compl);
	} else {
		return $maxUID;
	}
}

sub LastUID {
  my $this = shift;
  if ( @_ ) {
    my $UID = shift;
    if (($UID>$this->{'lastUID'}) && ($UID>$main::min_uid)){
      $this->{'lastUID'} = $UID;
      $this->{'lastUIDisUnChecked'} = 1;
    }
  }
  if ($this->{'checkLastUID'} && $this->{'lastUIDisUnChecked'}){
    $this->{'lastUID'} = &getFreeUID($this->{'lastUID'});
    $this->{'lastUIDisUnChecked'} = 0;
  }
  return $this->{'lastUID'};
}

sub getFreeUID {
  my $uid = shift;
  return undef unless ($uid=~/^\d+$/);
  my $login;
  while ($login = getpwuid($uid) ) {
    if ( $login =~ /^$::user_prefix\d+([p|f]\d+)?$/ ) {
      last; ## confixx-account
    }
    $uid++;
  }
  return $uid;
}

sub ReplaceUID {
  my $this = shift;
  my $srcUID = shift;
  my ($oldMin,$newMin,$dstUID);
  if ( $this->{'complicit'} ) {
    $dstUID = $this->{'lastUID'}+1;
    $srcUID = $this->LastUID($dstUID);
  } else {
    if ( ( $oldMin = $this->{'oldMinUID'} ) &&
				($newMin = $this->{'newMinUID'}||$this->{'last0UID'} ) ) {
      $dstUID = $srcUID-$oldMin+$newMin;
      if ( ($dstUID >= $newMin) && ($dstUID <= $this->{'newMaxUID'})){
				$srcUID = $dstUID;
      }
    }
  }
  if ($srcUID < $this->{'lastUID'} ) {
    $dstUID = &getFreeUID($srcUID);
    if ($dstUID != $srcUID) {
#
# matching of UIDs
#
      $srcUID = $dstUID;
      $this->{'complicit'} = 1;
    }
  } else {
    if ( $dstUID = $this->LastUID( $srcUID ) ) {
      $srcUID = $dstUID;
    }
  }
  return $srcUID;
}

sub CheckLastUID {
  my $this = shift;
  if ( @_ ) {
    $this->{'checkLastUID'} = shift;
  }
  return $this->{'checkLastUID'};
}

sub add_rule {
  my $this = shift;
  my ($name, $value) = @_;
  $$this{'rules'}{$name} = $value;
  return 1;
}

sub get_rule {
  my $this = shift;
  if ( @_ ) {
    my ($name) = @_;
    return $$this{'rules'}{$name};
  }else{
    return $$this{'rules'};
  }
}

sub getRuleHash {
  my $this = shift;
  return $$this{'rules'};
}

sub del_rule {
  my $this = shift;
  my ($name) = @_;
  my $val = $$this{'rules'}{$name};
  delete $$this{'rules'}{$name};
  return $val;
}

sub print_rule {
  my $this = shift;
  my $name = shift;
  my $ret = '';
  if ( exists($$this{'rules'}{$name} ) ) {
    my $val = $$this{'rules'}{$name};
    $ret = "$name\t=\trule(";
    if ( $val == $ASSIGN_CURRENT ) {
      $ret .= "assign_current)";
    } elsif ( $val == $RESTORE_DUMPED ) {
      $ret .= "restore_dumped)";
    } elsif ( $val == $APPEND_DUMPED ) {
      $ret .= "append_dumped)";
    }else{
      $ret.="$val)";
    }
  }
  return $ret;
}

sub add_attr {
  my $this = shift;
  my ($name, $value) = @_;
  $$this{'attrs'}{$name} = $value;
  return 1;
}

sub get_attr {
  my $this = shift;
  my ($name) = @_;
  my $ret = $$this{'attrs'}{$name};
  if ($ret=~/^value\(\s*(\S+)\s*\)$/){
    $ret=$1;
  }
  return $ret;
}

sub del_attr {
  my $this = shift;
  my $name = shift;
  my %ret;
  if ($name=~/\(\*\)\./){
    my ($left,$right)=split(/\*/,$name);
    $left=~s/\(/\\\(/g;
    $left=~s/\./\\\./g;
    $right=~s/\)/\\\)/g;
    $right=~s/\./\\\./g;
    my ($val,$key,$newVal,$newKey);
    while ( ($key,$val)=each(%{$this->{'attrs'}})){
      if ($right){
	if ($key=~/^$left(.*)$right$/){
	  $newKey=$1;
	  $newVal=$this->{'attrs'}->{$key};
	  $newVal=$1 if ($newVal=~/value\s*\(\s*(.*)\s*\)/);
	  $ret{$newKey}=$newVal;
	  delete $this->{'attrs'}->{$key};
	}
      }else{
	if ($key=~/^$left(.*)/){
	  $newKey=$1;
	  $newVal=$this->{'attrs'}->{$key};
	  $newVal=$1 if ($newVal=~/value\s*\(\s*(.*)\s*\)/);
	  $ret{$newKey}=$newVal;
	  delete $this->{'attrs'}->{$key};
	}
      }
    }
  }else{
    $ret{$name}=$this->{'attrs'}->{$name};
    delete $this->{'attrs'}->{$name};
  }
  return \%ret;
}

sub getAttrHash {
  my $this = shift;
  my $templ = shift;
  unless ($templ){
    return $this->{'attrs'};
  }

  my ($left,$right) = split(/\*/,$templ,2);
	my ($reParse);
	if ( $right ) {
		$reParse = qr/^\Q$left\E(.*)\Q$right\E$/;
	} else {
		$reParse = qr/^\Q$left\E(.*)/;
	}

  my ($ret,$val,$key,$newVal,$newKey);

	$ret = {}; ## empty anonymous hash

  while ( ($key,$val)=each(%{$this->{'attrs'}})){
		if ($key =~ /$reParse/){
			$newKey = $1;
			$newVal = $this->{'attrs'}->{$key};
			if ( $newVal =~ /value\s*\(\s*(.*)\s*\)/ ) {
				$newVal = $1;
			}
			$ret->{$newKey} = $newVal;
		}
  }
  return $ret;
}


sub stringify {
  my $this = shift;
  my $res_str = "Rules\n";
  foreach my $rule (keys  %{$$this{'rules'}}) {
    my $rule_val = &get_rule_name($$this{'rules'}{$rule});
    $res_str .= "$rule\t=>\t$$this{'rules'}{$rule}\n";
  }
  $res_str .= "Attrs:\n";
  foreach my $attr (keys %{$$this{'attrs'}}) {
    $res_str .= "$attr\t=>\t$$this{'attrs'}{$attr}\n";
  }
  return $res_str;
}

sub toString {
  my $this = shift;
  my $key = shift || 'all';
  my $emptyValue;
  if (@_){
    $emptyValue = shift ;
  }else{
    $emptyValue = '<insert new value>';
  }
  my ($res_str,$suffix,$rule_val,$rule,$attr,$value);
  my @sortedKeys;
  my ($section,$subsection,$oldSection,$oldSubsection);

  if ($key=~/rule|all/){
    $res_str .= "\nRules:\n\n" if ($key=~/all/);
    @sortedKeys = sort sortByKey (keys %{$$this{'rules'}});
    foreach $rule (@sortedKeys) {
      $rule_val = &get_rule_name($$this{'rules'}{$rule});
      $suffix='';
      if ($rule=~/(\S+)\(\s*\S*\s*\)\.(\S+)/){
	$section = $1;
	$subsection = $2;
	if(!($oldSection||$oldSubsection)){
	  $oldSection = $section;
	  $oldSubsection = $subsection;
	}elsif ($oldSection && $oldSection ne $section){
	  $suffix="\n";
	  $oldSection = $section;
	  $oldSubsection = $subsection;
	}elsif($oldSubsection && $oldSubsection ne $subsection){
	  $suffix="";
	  $oldSubsection = $subsection;
	}
      }
      unless(length($rule_val)){
	$rule_val = $emptyValue;
      }
      if ($rule_val){
	$res_str .= "$suffix$rule\t=\trule($rule_val)\n";
      }
    }
  }
  if ($key=~/attr|all/){
    $section = '';
    $subsection = '';

    $res_str .= "\nAttrs:\n" if ($key=~/all/);
    @sortedKeys = sort sortByKey (keys %{$$this{'attrs'}});
    foreach $attr (@sortedKeys) {
      next unless $attr;
      $suffix='';

      if ($attr=~/(\S+)\(\s*\S*\s*\)\.(\S+)/){
				$section = $1;
				$subsection = $2;

				if(!($oldSection||$oldSubsection)){
					$oldSection = $section;
					$oldSubsection = $subsection;

				}elsif($oldSection && $oldSection ne $section){
					$suffix = "\n\n";
					$oldSection = $section;
					$oldSubsection = $subsection;

				}elsif($oldSubsection && $oldSubsection ne $subsection){
					$suffix = "\n";
					$oldSubsection = $subsection;
				}
      }
      $value = $$this{'attrs'}{$attr};

      unless(length($value)){
	$value = $emptyValue;
      }
      if ($value){
	$res_str .= "$suffix$attr\t=\tvalue($value)\n";
      }
    }
  }
  return $res_str;
}

sub sortByKey {
  my ($keyA,$keyB);
  if ($Modules::Restore::Map::a =~ /(\S+)\(\s*(\S*)\s*\)\.(\S+)/){
    $keyA=pack ('A16A8A32',$1,$3,$2);
  }else {
    $keyA=$Modules::Restore::Map::a;
  }
  if ($Modules::Restore::Map::b=~/(\S+)\(\s*(\S*)\s*\)\.(\S+)/){
    $keyB=pack ('A16A8A32',$1,$3,$2);
  }else {
    $keyB=$Modules::Restore::Map::b;
  }
  $keyA cmp $keyB;
}
sub getKeyIndex {
  my $key = shift;
  my ($len,$k1,$k2,$k3);
  if ($key=~/(\S+)\(\s*(\S*)\s*\)\.(\S+)/){
    $key=pack ('A16A8A32',$1,$3,$2)
  }
  return $key;
}

sub get_rule_name() {
  my ($rule_val) = @_;
  if ($rule_val == $ASSIGN_CURRENT) {
    return 'assign_current';
  } elsif ($rule_val == $RESTORE_DUMPED) {
    return 'restore_dumped'; 
  }elsif ($rule_val == $APPEND_DUMPED) {
    return 'append_dumped'; 
  }
  return $rule_val;
}

sub defaultRule (){
  my $this=shift;
  if(@_){
    my $newValue=shift;
    if(($newValue==$APPEND_DUMPED)||($newValue==$RESTORE_DUMPED)||
       ($newValue==$ASSIGN_CURRENT)){
      $this->{'default'}=$newValue;
    }
  }
  return $this->{'default'};
}

sub parseSystem ($$;$){
  my $this = shift;
  my $file = shift;
	my $mapping = shift;
  return 0 unless (-T $file);

  my $xmlParser = new XML::DOM::Parser;
  my $xmlRoot = $xmlParser->parsefile($file);
  my ($kid,$xmlConfixx,$xmlSystem);
  foreach $kid ($xmlRoot->getChildNodes){
    next unless ($kid->getNodeType == ELEMENT_NODE);
    if ($kid->getNodeName() eq 'confixx'){
      $xmlConfixx=$kid;
      last;
    }
  }
  unless (defined($xmlConfixx)) {
    &errmsg("The XML config '$file' is malformed: no 'confixx' node found\n");
    return undef;
  }
  foreach $kid ($xmlConfixx->getChildNodes){
    next unless ($kid->getNodeType == ELEMENT_NODE);
    if ($kid->getNodeName() eq 'system'){
      $xmlSystem=$kid;
      last;
    }
  }
  unless (defined($xmlSystem)) {
    &errmsg("The XML config '$file' is malformed: no 'confixx/system' node found\n");
    return undef;
  }
  my (%sysFiles,$name);
  foreach $kid ($xmlSystem->getChildNodes){
    next unless ($kid->getNodeType == ELEMENT_NODE);
    $name =  $kid->getNodeName();
    $sysFiles{$name}=$kid;
  }
  
  my ($oldUID,$oldGID,$oldGroup,$newUID,$newGID,$newGroup,
      $oldUser,$newUser,%oldGIDs,$key);

#
# group
#
  if ( !$mapping && $sysFiles{'group'} ) {
    foreach $kid ($sysFiles{'group'}->getChildNodes ) {
      next unless ($kid->getNodeType == ELEMENT_NODE);
      next unless($kid->getNodeName == 'item');
      next unless ($name = $kid->getAttribute('name'));
      $oldGID = $kid->getAttribute('gid');
      $oldGIDs{$name} = $oldGID;
      
      next if $kid->getAttribute('login'); ## login, not group

      $name = "main::$name";
      next unless (defined($$name));

      $newGroup=$$name;
      next unless ($newGID = getgrnam($newGroup));
      $oldGroup= $kid->getAttribute('group');
      if ($newGroup && $oldGroup){
				$this->add_attr( "group($oldGroup).name", $newGroup );
      }
      if ($newGID && $oldGID){
				$this->add_attr( "group($oldGID).gid", $newGID );
      }
    }
  }
#
# password
#
  if (  !$mapping && $sysFiles{'passwd'} ) {
    foreach $kid ($sysFiles{'passwd'}->getChildNodes){
      next unless ($kid->getNodeType == ELEMENT_NODE);
      next unless($kid->getNodeName == 'item');
      next unless ($name = $kid->getAttribute('name'));
      $oldGID = $oldGIDs{$name};
      $name = "main::$name";
      next unless (defined($$name));

      $newUser = $$name;
      ($newUID,$newGID) = (getpwnam($newUser))[2,3];

      if ($newGID && $oldGID){
				$key = "group($oldGID).gid";
				unless ($this->get_attr($key)){
					$this->add_attr($key,$newGID);
				}
      }

      $oldUser = $kid->getAttribute('login');
      $oldUID = $kid->getAttribute('uid');
      if ($newUser && $oldUser){
				$this->add_attr( "user($oldUser).name", $newUser );
      }
      if ($newUID && $oldUID){
				$this->add_attr( "user($oldUID).uid", $newUID );
      }
    }
  }
#
# mysql
#
  if ( $sysFiles{'mysql'} ) {
    my ($Password,$verOld,$MajOld,$minOld,$Maj,$min);

    foreach $kid ($sysFiles{'mysql'}->getChildNodes){
      next unless ($kid->getNodeType == ELEMENT_NODE);
			$name = $kid->getNodeName;

      if ( ( $name eq 'item' ) && ( !$mapping ) ) {
				$oldUser = $kid->getAttribute('user') || $kid->getAttribute('User');
				$Password = $kid->getAttribute('password') || $kid->getAttribute('Password');

				$this->add_attr("mysql($oldUser).Password",$Password);

			} elsif ( $name eq 'server' ) {

				my $verOld = $kid->getAttribute('verstring');
				if ( $::mysqlUserVersion && $verOld ) {
					if ( $verOld =~ /^(\d+)\.(\d+)/ ){
						($majOld,$minOld) = ($1,$2);
					}
					if ( $::mysqlUserVersion =~ /^(\d+)\.(\d+)/ ){
						($maj,$min) = ($1,$2);
					}
					
					if ( ( $majOld < 4 && $maj < 4 ) || ## both version are less than 4
							 ( $majOld == $maj && $minOld == $min ) ) { 

						$this->add_rule( "mysql(*).Password", $RESTORE_DUMPED );
					}
				}
			}
		}
  }

  return 1;
}

1;
