package Modules::Restore::Mapper;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

# mapping
use locale;

use strict;
no strict "vars";

use Modules::Restore::Map;
use Modules::Restore::Service;
use Modules::Restore::Tables::Common;
use Modules::Restore::Tables::Table_admin;
use Modules::Restore::Tables::Table_domains;
use Modules::Restore::Tables::Table_ipadressen;


BEGIN {
  use Exporter;
  @ISA = qw(Exporter);
  @EXPORT_OK = qw(&parseMapFile &validateMapFile &gatherMapData &writeMapFile);
}

use Modules::Restore::Map qw(&get_rule_name);


# parse map file and remeber stored rules and attributes
# $file_name = string. name of a map file.
# $map       = object, class Map; map data container.

sub parseMapFile($$$) {
  my ($file_name, $map, $dumpDoc) = @_;
  

  die "Error of parsing of the map file: wrong arguments\n" 
    unless ((ref($map)=~/Map/) &&(ref($dumpDoc)=~/DumpDoc/));

  my $xmlConf=$dumpDoc->DumpConfXML;
  die "Can not get Modules::Restore::DumpConfXML object\n" 
    unless(ref($xmlConf)=~/DumpConfXML/);

  open FILE, "<$file_name"
    or die("Can not open file $file_name: $!\n");

  my ($tblName,$fldName,$oldval,$table);
  my $i_line = 0;
  while (<FILE>) {
    my $cur_line = $i_line++;
    if        ($_ =~ m/^\s*#/) {
			# comment
      next;
    } elsif   ($_ =~ m/^\s*$/) {
			# empty line
      next;
    } elsif   ($_ =~ m/^\s*([\d\w\(\)\*\.\-\x81-\xFE]+)\s*=\s*(.+)(\s*\#.*)?$/ ) {
			# parameter assignment
      my ($var_str, $val_str) = ($1, $2);
      my @var_items = split /./, $var_str;
      if      ($val_str =~ m/rule\((\S*)\)/) {
	my ($rule_str, $rule_val) = ($1, $1);
	if      ($rule_str eq 'restore_dumped') {
	  $rule_val = $RESTORE_DUMPED;
	} elsif ($rule_str eq 'assign_current') {
	  $rule_val = $ASSIGN_CURRENT;
	} elsif ($rule_str eq 'append_dumped') {
	  $rule_val = $APPEND_DUMPED;
	}

	$map->add_rule($var_str, $rule_val);
      } elsif ($val_str =~ m/value\(\s*(.+)\s*\)/) {
	$val_str=$1;
	if ($val_str=~/^<.*\w+\s+\w+.*>$/){ # <insert new value>
	  $val_str=undef;
	}
	$map->add_attr($var_str, $val_str);
	if ($var_str =~ /(\S+)\(\s*(\S+)\s*\)\.(\S+)/){
	  $tblName = $1;
	  $oldValue = $2;
	  $fldName = $3;
	  if ($tblName eq 'domain' && ($fldName =~ /name|standard/ )){
	    if($table=$xmlConf->getTable('domains')){
	      if (($oldValue=~/^([\da-zA-Z0-9\-]+\.)+[a-zA-Z]+$/) && 
		  ($val_str=~/^([\da-zA-Z0-9\-]+\.)+[a-zA-Z]+$/)){
		
		$table->addToMap('domain',$oldValue,$val_str);

	      }
	    }
	  }elsif($tblName eq 'ip' && ($fldName =~ /address/)){
	    if($table=$xmlConf->getTable('ipadressen')){
	      if (($oldValue=~/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) && 
		  ($val_str=~/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)){
		
		$table->addToMap('ip',$oldValue,$val_str);

	      }
	    }
	  }
	  elsif($table=$xmlConf->getTable($tblName)){ ## check table
	    if ($fldName eq 'id'){
	      $fldName = $table->AttrID;
	    }
	    if($table->getFieldAttr($fldName)){ ## check field
	      if ($oldValue && $val_str){
		$table->addToMap($fldName,$oldValue,$val_str);
	      }
	    }
	  }
	}
      } else {
	die("Parse error in '$file_name', line $cur_line: '$val_str' is incorrect right part\n");
      }
    } else {
      die("Parse error in '$file_name', line $cur_line: $_\n");
    }
  }
  close FILE
    or die("Can not close file $file_name: $!\n");

  return 1;
}

# check that all required data are specified in map 
sub validateMapFile($$) {
  my $map = shift;
  my $dumpDoc =shift;

  unless (ref($map)=~/Map/){
    print STDERR "Mapper::validateMapFile : error wrong type of argument\n";
    return 0;
  }
  unless (ref($dumpDoc)=~/DumpDoc/){
    print STDERR "Mapper::validateMapFile : error wrong type of argument\n";
    return 0;
  }

  my $ptrIPs=$map->getAttrHash('ip(*).address');
  my ($oldIP,$newIP);
  my $ret=1;
  if (ref($ptrIPs)=~/HASH/){
    while (($oldIP,$newIP)=each(%{$ptrIPs})){
      unless (($oldIP=~/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/) &&
	      ($newIP=~/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/)){
	print STDERR "Bad ip address(-es): $oldIP => $oldIP\n";
	$ret=0;
      }
    }
  }

  my $ptrDoms=$map->getAttrHash('domain(*).name');
  my $clean= $dumpDoc->AttrClean;
  my %domains;

  if ($clean || 
      ($map->get_rule('domain(*).standard') == $RESTORE_DUMPED)){

    my $ptrStd=$map->getAttrHash('domain(*).standard');
    my ($oldDom,$newDom);
    while (($oldDom,$newDom)=each(%{$ptrStd})){
      unless ($newDom=~/([\da-zA-Z\-]+\.)+[\da-zA-Z\-]+/){
	print STDERR "Map error: Bad domain name: $oldDom => $newDom\n";
	$map->del_attr("domain($oldDom).standard");
	$ret=0;
	next;
      }
      $ptrDoms->{$oldDom}=$newDom;
    }
  }

  unless($clean){
    my $sth=$::dbh->prepare('select domain from domains');
    $sth->execute;
    while (($newDom)=$sth->fetchrow){
      $domains{$newDom}=1;
    }
    $sth->finish;

  }

  while (($oldDom,$newDom)=each(%{$ptrDoms})){
    next unless $newDom;
    unless ($newDom=~/([\da-zA-Z\-]+\.)+[\da-zA-Z\-]+/){
      print STDERR "Map error: Bad domain name: $oldDom => $newDom\n";
      $map->del_attr("domain($oldDom).name");
      $ret=0;
      next;
    }
    if (exists($domains{$newDom})){
      print STDERR "Map error: Already there are domain name: $oldDom => $newDom\n";
      $ret=0;
    }
  }

  my $ptrHash=$map->getAttrHash;
  while (my($key,$val)=each(%{$ptrHash})){
    next unless $key;
    unless (defined($val)){
      print STDERR "Map error: Map has the undefined value of attribute : $key = $val\n";
      $ret=0;
      next;
    }
    $val=~s/value\s*\(\s*(.*)\s*\)/$1/;
    if ($val=~/^<.*\w+\s+\w+.*>$/){
      print STDERR "Map error: Map has the undefined value of attribute : $key\t=\tvalue($val)\n";
      $ret=0;
      next;
    }
  }

  return $ret;
}

# walk through map and gather data to map
# $dump_doc = object, class Dump. dump file parser.
sub gatherMapData($$) {
  my ($dump_doc,$objMap) = @_;
	
  $dump_doc->setTableHandler
    (
     'ipadressen' => 
     {
      'pre_table' => \&Modules::Restore::Tables::Table_ipadressen::pre_map_tab,
      'on_row' => \&Modules::Restore::Tables::Table_ipadressen::map_row
     },
     'admin'	  => 
     {
      'pre_table' => \&Modules::Restore::Tables::Table_admin::pre_tab,
      'on_row' => \&Modules::Restore::Tables::Table_admin::map_row,
     },
     'domains' =>
     {
      'pre_table' => \&Modules::Restore::Tables::Table_domains::pre_map_tab,
      'on_row' => \&Modules::Restore::Tables::Table_domains::map_row,
      'post_table'=> \&Modules::Restore::Tables::Table_domains::post_map_tab,
     },
     'anbieter' =>
     {
      'pre_table' => \&Modules::Restore::Tables::Common::nothing,
      'on_row' =>  \&Modules::Restore::Tables::Common::nothing,
     },
     'kunden' =>
     {
      'pre_table' =>  \&Modules::Restore::Tables::Common::nothing,
      'on_row' =>  \&Modules::Restore::Tables::Common::nothing,
     },
     'pop3' =>
     {
      'pre_table' =>  \&Modules::Restore::Tables::Common::nothing,
      'on_row' =>  \&Modules::Restore::Tables::Common::nothing,
     },
     'ftp' =>
     {
      'pre_table' =>  \&Modules::Restore::Tables::Common::nothing,
      'on_row' =>  \&Modules::Restore::Tables::Common::nothing,
     },
     'mysql_datenbanken' =>
     {
      'pre_table' =>  \&Modules::Restore::Tables::Common::nothing,
      'on_row' =>  \&Modules::Restore::Tables::Common::nothing,
     },
     'skins' =>
     {
      'pre_table' =>  \&Modules::Restore::Tables::Common::nothing,
      'on_row' =>  \&Modules::Restore::Tables::Common::nothing,
     },

     '*' =>
     {
      'pre_table' => \&Modules::Restore::Tables::Common::pre_map_tab,
      'on_row' => \&Modules::Restore::Tables::Common::map_row
     }
    );
  $dump_doc->processDump();
}

#
# Set of mapping routines
#

my @ip_used = ();
my $std_ip  = '';





#
# Save map file
#

# save map file using found data to map
# $file = string. file name where a map should be saved.
sub writeMapFile($) {
  my ($file,$clean,$isClean,$licenceOK) = @_;

  my (%printedRule,$key,$ptrRules);
   open FILE, ">$file"
    or die("Can not write to $file: $!\n");

  my $defaultRule=$main::map->defaultRule;
  $defaultRule=$clean?$RESTORE_DUMPED:$ASSIGN_CURRENT  unless ($defaultRule);

  &printComments(*FILE,'main.begin');

  &printComments(*FILE,'licence_problem') unless $licenceOK;

  &printComments(*FILE,'main.isnotclean') unless $isClean;
  &printComments(*FILE,'main.clean') if $clean;

  &printComments(*FILE,'network.begin');

  $key='admin(*).dnstemplate';
  &printComments(*FILE,$key);
  $printedRule{$key} = $defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  $key='admin(*).updateinterval';
  &printComments(*FILE,$key);
  $printedRule{$key} = $defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  $key='reseller(*).dnstemplate';
  &printComments(*FILE,$key);
  $printedRule{$key} = $defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  $key='reseller(*).pns';
  &printComments(*FILE,$key);
  $printedRule{$key}=$ASSIGN_CURRENT;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";
  $key='reseller(*).sns';
  $printedRule{$key}=$ASSIGN_CURRENT;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  $key='user(*).default';
  &printComments(*FILE,$key);
  $printedRule{$key} = $defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  $key='reseller(*).default';
  &printComments(*FILE,$key);
  $printedRule{$key} = $defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";


  $key='domain(*).dnszone';
  &printComments(*FILE,$key);
  $printedRule{$key} = $defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";
  $key='domain(*).standard';
  $printedRule{$key}=$ASSIGN_CURRENT;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";


	# admin.standarddomain -- use current, modify domains (richtigedomain = 3,5)
	# admin.hostname -- use current
	# admin.localeips -- map via ipadressen
	# admin.standardip -- use current
	# reseller.standardip -- suggest current, map (via ipaddressen)
	# ipaddressen -- map

  &printComments(*FILE,'ip(*).address');
  my $ptrAtrrs =$main::map->del_attr('ip(*).address');
  if($ptrAtrrs){
    my($ip,$newValue);
    while ( ($ip,$newValue) =each (%{$ptrAtrrs})) {
      my $std_str = ($ip eq $std_ip) ? "\t#Standard IP" : "";
      if ($newValue){
	print FILE "ip($ip).address = value($newValue)$stdstr\n";
      }else{
	print FILE "ip($ip).address = value(<Your IP>)$stdstr\n";
      }
    }
  }
	# kunden.ip -- map via ipaddressen

  &printComments(*FILE,'domain(*).standard');
  $ptrAtrrs =$main::map->del_attr('domain(*).standard');
  if($ptrAtrrs){
    my($stddom,$newValue);
    while ( ($stddom,$newValue) = each (%{$ptrAtrrs})) {
      if ( $newValue ) {
				print FILE "\ndomain($stddom).standard = value($newValue)\n\n";
      } else {
				print FILE "\ndomain($stddom).standard = value(<Your standard domain>)\n\n";
      }
    }
  }

  my $str;
  $key='domain(*).default';
  if ($str=$main::map->print_rule($key)){
    $printedRule{$key}=$main::map->get_rule($key);

    &printComments(*FILE,$key);
    print FILE $str,"\n";
  }


  my($oldDom,$newDom);


  &printComments(*FILE,'domain(*).name');

  $ptrAtrrs = $main::map->del_attr('domain(*).suffix');
  while ( ($oldDom,$newDom)=each (%{$ptrAtrrs})){
    print FILE "#domain($oldDom).suffix\t=\tvalue($newDom)\n";
  }
  print FILE "\n";

  $ptrAtrrs =$main::map->del_attr('domain(*).name');
  while ( ($oldDom,$newDom)=each (%{$ptrAtrrs})){

    next if ($newDom);
		if ( $clean ) {
      print FILE "#domain($oldDom).name\t=\tvalue($oldDom)\n";
		} else {
      print FILE "domain($oldDom).name\t=\tvalue(<insert new value>)\n";
		}
  }

  &printComments(*FILE,'domain(*).name.defined');

  while (($oldDom,$newDom)=each(%{$ptrAtrrs})){
    next unless ($newDom);
    print FILE "#domain($oldDom).name\t=\tvalue($newDom)\n";
  }

  $ptrAttrs =$main::map->del_attr('service(*).state');
  if ((ref($ptrAttrs)=~/HASH/) && (keys %{$ptrAtrrs})){

    &printComments(*FILE,'service(*).state');

    while (my($servName,$servVal)=each(%{$ptrAttrs})){
      print FILE "service($servName).state\t=\tvalue($servVal)\n";
    }
  }

  my $srvMysql = $::objDumpConf->getService('mysql');
  unless(ref($srvMysql)=~/Service/ && $srvMysql->isStateOff){
    $key='mysql(*).Password';
		$rule = $main::map->get_rule( $key );
    &printComments( *FILE, $key );

    $printedRule{$key} = $rule || $RESTORE_DUMPED;

		print FILE '#' unless defined $rule;
		
    print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  }

  $key='skins(*).default';
  &printComments(*FILE,$key);

  $printedRule{$key}=$defaultRule;
  print FILE "$key\t=\trule(".&get_rule_name($printedRule{$key}).")\n";

  &printComments(*FILE,'file(*).default');

  my ($secName,$section,$rule,$defRule);
  foreach $secName ($::objDumpConf->getSectionsList){
    $section=$::objDumpConf->getSection($secName);
    next unless (ref($section)=~/Section/);
    $rule=$section->Attribute('rule');
    next unless $rule=~/^\S+\(\*\)\.\S+$/;
    next if(exists($printedRule{$rule}));
    $defRule = $defaultRule;
    unless( $defRule = $::map->get_rule( $rule ) ) {
      if ($secName =~ /user|mailbox|spoolbox|maildir|majordomo/){
				$defRule = $RESTORE_DUMPED;
      } else {
				$defRule = $defaultRule;
      }
    }
    $printedRule{$rule} = $defRule;
    print FILE "$rule\t=\trule(".&get_rule_name($printedRule{$rule}).")\n";

  }

  &printComments(*FILE,'rule(*).other');

#
# clean already printed rules
#
  foreach $key (keys %printedRule){
    $::map->del_rule($key);
  }

#
# clean idle rules
#
  $ptrRules=$::map->getRuleHash;
  my $tblName;
  foreach $key (keys %{$ptrRules}){
    next unless $key=~/^(\S+)\(\*\)\.default$/;
    $tblName=$1;
    $ptrAttrs=$::map->getAttrHash($tblName.'(*');
    unless (keys %{$ptrAttrs}){
      $::map->del_rule($key);
    }
  }
#
# print rules
# 
  print FILE $main::map->toString('rule');


  $ptrAtrrs =$main::map->del_attr('user(*).prefix');

  foreach $key ('user(*).uid','user(*).name','group(*).gid','group(*).name'){
    $::map->del_attr($key);
  }


  &printComments(*FILE,'attribute(*).other');

  print FILE $main::map->toString('attr');

  &printComments(*FILE,'main.end');

  close FILE
    or die("Can not save data to $file: $!\n");
		
  return 1;
}

sub printComments {
  my ($fh,$key)=@_;
  my $text = &::ltext($key);
  my $lf="\n";
  my @strs = split($lf,$text);
  print $fh $lf;
  foreach my $str (@strs){
    print $fh "#$str\n";
  } 
  print $fh $lf;
}

1;
