package Modules::Restore::Table;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

BEGIN {
  use Exporter;
  @ISA = qw(Exporter);
  @EXPORT = qw(&new &addToMap &getMap &getMappedValue &getLinks 
	       &hasAnyAttr &hasAllAttr &Attribute &AttrID &AttrOrder &AttrNoClean
	       &AttrIP &AttrNew & AttrDel &TypeID &MaxID &getNextID
	       &State &StateClean &addField
	       &fillAttributes &fillFields &getFieldAttr &getFieldAttrSet
	       &getFields &getName &addProp &getManipulator &AttrRule );	
}


use XML::DOM;

sub new {
  my $classname = shift;
  my ($name, $manipulator) = @_;
  my $this = {
	      'name'  		=> $name,
	      'manipulator'	=> $manipulator,
	      'props' 		=> {},
	      'fields'=>{},
	      'attrs'=>{},
	      'states'=>{},
	      'links'=>{},
	      'map'=>{},
	     };
  if($name){
    $this->{'attrs'}->{'rule'}=$name.'(*).default';
  }
  return bless ($this, $classname);
}

sub addToMap($$$){
  my $this = shift;
  my $mapName = shift;
  return undef unless $mapName;
  my ($oldValue,$newValue)=@_;
  $this->{'map'}->{$mapName}->{$oldValue}=$newValue;
  return $this->{'map'}->{$mapName};
}

sub getMap($){
  my $this = shift;
  my $mapName = shift;
  return $this->{'map'}->{$mapName};
}

sub getMappedValue($$){
  my $this = shift;
  my ($mapName,$oldValue) = @_;
  
  if(exists($this->{'map'}->{$mapName})){
    if(exists($this->{'map'}->{$mapName}->{$oldValue})){
      $oldValue=$this->{'map'}->{$mapName}->{$oldValue};
      return $oldValue;
    }
  }
  return undef;
}


sub getLinks(){
  my $this = shift;
  if (@_){
    my $table = shift;
    if (exists($this->{'links'}->{$table})){
      if (@_){
	my $field = shift;
	foreach my $lnkFld (@{$this->{'links'}->{$table}}){
	  if ($lnkFld eq $field){
	    return 1;
	  }
	}
	return 0;
      }else{
	return \@{$this->{'links'}->{$table}};
      }
    }
  }else{
    return $this->{'links'};
  }
  return undef;
}

sub hasAnyAttr (){
  my $this = shift;
  foreach my $key (@_){
    return $key if exists($this->{'attrs'}->{$key});
  }
  return 0;
}

sub hasAllAttr (){
  my $this = shift;
  foreach my $key (@_){
    return 0 unless exists($this->{'attrs'}->{$key});
  }
  return 1;
}

sub Attribute(){
  my $this = shift;
  my $name = shift;
  unless ($name){
    return undef;
  }
  if(@_){
    $this->{'attrs'}->{$name}=shift;
  }
  return $this->{'attrs'}->{$name};
}
sub AttrID(){
  my $this = shift;
  if(@_){
    my $fldName=shift;
    if (exists($this->{'fields'}->{$fldName})){
      $this->{'attrs'}->{'id'}=$fldName;
      $this->{'attrs'}->{'idtype'}=$this->{'fields'}->{$fldName}->{'type'};
    }else{
      return undef;
    }
  }
  return $this->{'attrs'}->{'id'};
}

sub AttrOrder(){
  my $this = shift;
  if(@_){
    my $fldNames=shift;
    my @flds=split(/\s*,\s*/,$fldNames);
    foreach my $fldName (@flds){
      unless (exists($this->{'fields'}->{$fldName})){
	return undef;
      }
    }
    $fldNames=join(',',@flds);
    $this->{'attrs'}->{'order'}=$fldNames;
  }
  return $this->{'attrs'}->{'order'};
}

sub TypeID(){
  my $this=shift;
  return $this->{'attrs'}->{'idtype'};
}

sub MaxID(){
  my $this=shift;
  if ($this->{'attrs'}->{'idtype'}=~/I/){
    if (@_){
      my ($newMax,$noCheck) = @_;
      if ($newMax=~/^\d+$/){
	if (!$noCheck && exists($this->{'attrs'}->{'idmax'})){
	   $this->{'attrs'}->{'idmax'}=$newMax if ($this->{'attrs'}->{'idmax'}<$newMax);
	}else{
	  $this->{'attrs'}->{'idmax'}=$newMax;
	}
      }
    }
    return $this->{'attrs'}->{'idmax'};
  }else{
    return undef;
  }
}

sub getNextID(){
  my $this=shift;
  if ($this->{'attrs'}->{'idtype'}=~/I/){
    if (exists($this->{'attrs'}->{'idmax'})){
      $this->{'attrs'}->{'idmax'}++;
    }else{
      $this->{'attrs'}->{'idmax'}=1;
    }
    return $this->{'attrs'}->{'idmax'};
  }else{
    return undef;
  }
}

sub AttrNoClean(){
  my $this = shift;
  if(@_){
    $this->{'attrs'}->{'noclean'} = shift;
  }
  return $this->{'attrs'}->{'noclean'};
}

sub AttrRule(){
  my $this = shift;
  if(@_){
    $this->{'attrs'}->{'rule'} = shift;
  }
  return $this->{'attrs'}->{'rule'};
}

sub State(){
  my $this = shift;
  my $name = shift;
  unless ($name){
    return undef;
  }
  if(@_){
    $this->{'states'}->{$name}=shift;
  }
  return $this->{'states'}->{$name};
}


sub StateClean(){
  my $this = shift;
  if(@_){
    $this->{'states'}->{'clean'} = shift;
  }
  return $this->{'states'}->{'clean'};
}

sub AttrIP(){
  my $this = shift;
  if(@_){
    $this->{'attrs'}->{'ip'} = shift;
  }
  return $this->{'attrs'}->{'ip'};
}

sub AttrNew(){
  my $this = shift;
  if(@_){
    $this->{'attrs'}->{'new'} = shift;
  }
  return $this->{'attrs'}->{'new'};
}

sub AttrDel(){
  my $this = shift;
  if(@_){
    $this->{'attrs'}->{'del'} = shift;
  }
  return $this->{'attrs'}->{'del'};
}

sub AttrUID(){
  my $this = shift;
  if(@_){
    $this->{'attrs'}->{'uid'} = shift;
  }
  return $this->{'attrs'}->{'uid'};
}

sub addField(){
  my $this = shift;
  my $fldName = shift;
  return 0 unless ($fldName);
  my $attrs=shift;
  my $field;
  unless(exists($this->{'fields'}->{$fldName})){
    $this->{'fields'}->{$fldName}={};
    $field=$this->{'fields'}->{$fldName};
    $field->{'name'}=$fldName;
  }else{
    $field=$this->{'fields'}->{$fldName};
  }
  while(my($key,$value)=each(%{$attrs})){
    $field->{$key}=$value;
    if ($key eq 'type'){
      if ( $value eq 'ip'){
	$this->{'attrs'}->{'ip'}.="$fldName,";
      }elsif($value eq 'account'){
	$this->{'attrs'}->{'account'}.="$fldName,";
      }elsif($value eq 'uid'){
	$this->{'attrs'}->{'uid'}.="$fldName,";
      }elsif($value eq 'url'){
	$this->{'attrs'}->{'url'}.="$fldName,";
      }
    }elsif ($key eq 'new'){
      $this->{'attrs'}->{'new'}.="$fldName,";
    }elsif ($key eq 'del'){
      $this->{'attrs'}->{'del'}.="$fldName,";
    }elsif ($key eq 'unique'){
      $this->{'attrs'}->{'unique'}.="$fldName,";
    }elsif ($key eq 'buildin'){
      if($this->{'attrs'}->{'buildin'}){
	$this->{'attrs'}->{'buildin'}.=' and ';
      }
      $this->{'attrs'}->{'buildin'}.="($fldName = '$value')";
    }
  }
}
 
sub fillAttributes(){
  my $this = shift;
  my $node = shift;
  my $cnt=0;
  unless(ref($node)=~/XML/i){
    return 0;
  }
  unless( $node->getNodeType == ELEMENT_NODE){
    return 0;
  }
  my($tblAttrs,$attr,$attrName,$attrValue,$attrs);

#
#  read tables attributes
#
  if (my $tblAttrs=$node->getAttributes()){
    for  $attr (@{$tblAttrs->getValues}){
      if ($attr->isSpecified){
	$attrName=$attr->getName();
	unless($attrName =~ /^safe|dump$/){
	  $attrValue=$attr->getValue();
	  $this->{'attrs'}->{$attrName}=$attrValue;
	  $cnt++;
	}
      }
    }
  }
#
# /tables attributes
#
  return $cnt;
}

sub fillFields(){
  my $this = shift;
  my $nodeTable = shift;
  unless(ref($nodeTable)=~/XML/i){
    return 0;
  }
  unless( $nodeTable->getNodeType == ELEMENT_NODE){
    return 0;
  }
  my $fields=$this->{'fields'};
  my ($nodes,$cntNodes,$i);
  my($fldAttrs,$attr,$attrName,$attrValue,$attrs,$lnkTable,$lnkField,$fldName,$oldAttr);
  my $tblAttrs=$this->{'attrs'};;

  my $dbIsPg=($::dbType eq 'Pg');
#
# fields
#
  foreach $node ($nodeTable->getChildNodes()){
    next unless ($node->getNodeType==ELEMENT_NODE);
    next unless ($node->getNodeName() eq 'field');
    unless($fldName = $node->getAttribute('name')){
      next;
    }
    if ($dbIsPg){
      $fldName = &::toLowCase($fldName ,$node,'name');
    }
    if(exists($fields->{$fldName})){
      print STDERR "WARNING: There is already field '$fldName'\n";
    }else{
      $fields->{$fldName}={};
    }
    $attrs=$fields->{$fldName};
    if ($fldAttrs=$node->getAttributes()){
      for  $attr (@{$fldAttrs->getValues}){
	if ($attr->isSpecified){
	  $attrName=$attr->getName();
	  $attrValue=$attr->getValue();
	  $attrs->{$attrName}=$attrValue;
	  if ($attrName eq 'type'){
	    if ($attrValue eq 'ip'){
	      $tblAttrs->{'ip'}.="$fldName,";
	    }elsif ($attrValue eq 'uid'){
	      $tblAttrs->{'uid'}.="$fldName,";
	    }elsif ($attrValue eq 'account'){
	      $tblAttrs->{'account'}.="$fldName,";
	    }elsif ($attrValue eq 'url'){
	      $tblAttrs->{'url'}.="$fldName,";
	    }
	  }
	  if ($attrName eq 'new'){
	    $tblAttrs->{'new'}.="$fldName,";
	  }elsif ($attrName eq 'del'){
	    $tblAttrs->{'del'}.="$fldName,";
	  }elsif ($attrName eq 'unique'){
	    $tblAttrs->{'unique'}.="$fldName,";
	  }elsif ($attrName eq 'link'){
	    if($dbIsPg){
	      $attrValue=&::toLowCase($attrValue,$node,$attrName);
	    }
	    ($lnkTable,$lnkValue)=split(/\./,$attrValue);
	    if($lnkTable){
	      push @{$this->{'links'}->{$lnkTable}},$lnkValue;
	      $this->Attribute('links',1);
	    }
	  }elsif ($attrName eq 'buildin'){
	    if ($tblAttrs->{'buildin'}){
	      $tblAttrs->{'buildin'}.=' and ';
	    }
	    $tblAttrs->{'buildin'}.="($fldName = '$attrValue')";
	  }
	}
      }
    }
  }
#
# /fields
#
  return $cntNodes;
}

sub getFieldAttr(){
 my $this = shift;
 my $fldName = shift;
 my $attrName = shift;
 if (exists($this->{'fields'}->{$fldName})){
   return $this->{'fields'}->{$fldName}->{$attrName};
 }else{
   return undef;
 }
}

sub getFieldAttrSet(){
 my $this = shift;
 my $fldName = shift;
 if(exists($this->{'fields'})){
   return  $this->{'fields'}->{$fldName};
 }else{
   return undef;
 }
}

sub getFields(){
 my $this = shift;
 return $this->{'fields'};
}

sub getName() {
  my $this = shift;
  return $$this{'name'};
}

sub isPropSafe($) {
  my $this = shift;
  my ($name) = @_;
  return undef unless (exists $$this{'props'}{$name});
  return $$this{'props'}{$name}{'safe'};
}

sub addProp($%) {
  my $this = shift;
  my ($name, %attrs) = @_;
  $$this{'props'}{$name} = {%attrs};
  return 1;
}

sub getManipulator() {
  my $this = shift;
  return $$this{'manipulator'};
}

1;
