package Modules::Restore::Tables::Common;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####



use DBI;


use XML::DOM;
use XML::DOM::NodeList;

use vars qw( %RsrvdLink $olduserPrefix $dbQuotes);

BEGIN {
  use Exporter;
  @ISA = qw(Exporter);
  @EXPORT = qw(&insert_query($$) &update_query($$$) &clean_tab($$) &restore_row($$$)
	       &pre_restore_tab($$$$) &post_restore_tab($$$$) &oneline_restore($$$) );	

  %RsrvdLink =('anbieter.anbieter' => 1,
	     'kunden.kunde' => 1 ,
	     'domains.domain' => 1);

  $oldUserPrefix='';

}


use strict;


use Modules::Restore::Map;
use Modules::Restore::Service;
use Modules::Restore::Table;
use Modules::Restore::DumpConfXML;
use Modules::Restore::DumpDoc;

use Modules::Restore::Tables::Table_kunden;
use Modules::Restore::Tables::Table_domains;

use Modules::Restore::Tables::Table_domains qw(&getNewDomain($));
use Modules::Restore::Tables::Table_kunden qw($oldUser0 $newUser0);
use Modules::Restore::Map qw($RESTORE_DUMPED $ASSIGN_CURRENT);


sub insert_query($$) {
  my ($table_name, $ptrAttrs) = @_;

  unless (ref($ptrAttrs)=~/HASH/){
    die "Modules::Restore::Tables::Common::insert_query: arguments error\n";
    return; 
  }

  my (@fields, @values, @templs);
  while (my ($attr, $val) = each (%{$ptrAttrs})) {
    if($attr && defined($val)){
      push @fields, $::dbQuotes? $::dbQuotes.$attr.$::dbQuotes: $attr;
      push @values, $val;
      push @templs, '?';
    }else{
      warn "Value of $attr column in table $table_name is undefined\n";
    }
  }
  my $fields_str = join ',', @fields;
  my $values_str = join ',', @values;
  my $templs_str = join ',', @templs;
	
  my $query = "INSERT INTO $table_name ($fields_str) VALUES ($templs_str)";
  my $sth = $main::dbh->prepare($query);
  $sth->execute(@values)
    or die("Can not execute query '$query' with values (@values): $DBI::error\n");
  if ($::DEBUG){
    print STDERR "SQL: $query (@values)\n" if $::DEBUG>2;
#    print STDERR "SQL: insert into $table_name ...\n";
  }
	
}

sub update_query($$$) {
  my ($table_name, $ptrAttrs,$where) = @_;
  unless (ref($ptrAttrs)=~/HASH/){
    die "Modules::Restore::Tables::Common::update_query: arguments error\n";
    return; 
  }

  my (@fields, @values, @templs);
  while (my ($attr, $val) = each (%{$ptrAttrs})) {
    if ($attr && defined($val)){
      push @values, $val;
      push @templs,$::dbQuotes?"$::dbQuotes$attr$::dbQuotes=?":"$attr=?";
    }
  }
  my $templs_str = join ', ', @templs;

  my $query = "UPDATE $table_name SET $templs_str";
  if($where){
    $where=~s/^\s*//;
    $where=~s/\s*$//;

    unless ($where=~/^where[\s|\(]/i){
      $where=" where ($where)";
    }
    $query.=" $where";
  }
  my $sth = $main::dbh->prepare($query);
  $sth->execute(@values)
    or die("Can not execute query '$query' with values (@values): $DBI::errstr\n");
  if ($::DEBUG>2){
    print "SQL: $query (@values)\n"  ;
#    print STDERR "SQL: update $table_name ...\n";
  } 
}

sub clean_tab($$){
  my ($table_name, $ptrConfXML) = @_;
  unless ($table_name && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Modules::Restore::Tables::Common::clean_tab : wrong arguments ($table_name,..)\n";
    return 1;
  }
  my ($table,$tblName,$fldName,$ptrField,$link,$lnkTable,$lnkField,$ptrFields);
  unless($table=$ptrConfXML->getTable($table_name)){
    return 1;
  }
  if ($table->AttrNoClean){
#    $table->StateClean(1);
    return 0;
  }

  return 0 if $table->StateClean;

  my (%toClean,$tblClean);
  $toClean{$table_name}=$table;

  my (%toWork,$loop);
  foreach  $tblName  ($ptrConfXML->getTablesList){
    unless($table=$ptrConfXML->getTable($tblName)){
      next;
    }
    unless ($table->StateClean || $table->AttrNoClean){
      $toWork{$tblName}=$table;
      $loop=1;
    }
  }

#
# calculate list of the linked tables
#
  while ($loop){
    $loop=0;
    while (($tblName,$table) =  each(%toWork)) {
      next if $table->StateClean;
      foreach $tblClean (keys %toClean){
	if ($table->getLinks($tblClean)){
	  $toClean{$tblName}=$table;
	  delete $toWork{$tblName};
	  $loop=1;
	  last;
	}
      }
    }
  }

#
# clean tables
#
  my ($sql,$where);
  my $userCurrent = ($::map->get_rule('user(*).default')==$ASSIGN_CURRENT);
  my $resellerCurrent = ($::map->get_rule('reseller(*).default')==$ASSIGN_CURRENT);
  my $user0 = $Modules::Restore::Tables::Table_kunden::oldUser0 || $::user_prefix.'0';
  my ($fldID,$sth,$maxID,$seqName);

  while (($tblName,$table) = each (%toClean)){
    print STDERR "Clean table $tblName\n" if ($::DEBUG);

    if(my $fldList=$table->AttrDel){
      my ($delValue,@set);
      my @flds = split(/,/,$fldList);
      
      foreach $fldName (@flds){
	next unless $fldName;
	$delValue = $table->getFieldAttr($fldName,'del')||"'1'";
	push @set,"$fldName=".$delValue;
      }
      if(@set){
	$sql="update $tblName set ".join(',',@set);
      }
    }else{
      $sql="delete from $tblName";
    }
    $where='';
    if ($tblName eq 'kunden'){
      $where="(kunde != '$user0')";
    }elsif($tblName eq 'anbieter'){
      $where="(anbieter != 'res0')";
    }else{
      if($table->Attribute('buildin')){
	$where.=' and ' if $where;
	$where=$table->Attribute('buildin');
      }
      if (($userCurrent && $table->getLinks('kunden','kunde')) || 
	  ($resellerCurrent && $table->getLinks('anbieter','anbieter'))
	 ){
	$ptrFields=$table->getFields;
	while (($fldName,$ptrField) = each(%{$ptrFields})){
	  if($link=$ptrField->{'link'}){
	    if ($link=~/anbieter\.anbieter/){
	      if ($resellerCurrent){
		$where .= ' and ' if $where;
		$where .= "($fldName!='res0')";
	      }
	    }elsif ($link=~/kunden\.kunde/){
	      if ($userCurrent){
		$where .= ' and ' if $where;
		$where .= "($fldName!='$user0')";
	      }
	    }
	  }
	}
      }
    }
    if ($where){
      $sql.=' where '.$where;
    }

    $main::dbh->do($sql) or
      print STDERR "Error: Tables::Common: execute SQL-request: $sql\n$::dbh->errstr\n";

    if (($table->TypeID=~/^I/) && ($fldID=$table->AttrID)){
#
# recalc maximum of ID (if it is numeric)
#
      if ($where){
	$sql="select max($fldID) from $tblName";
	$sth=$::dbh->prepare($sql);
	if($sth->execute && $sth->rows){
	  ($maxID)=$sth->fetchrow;
	  $sth->finish;
	}
      }else{
	$maxID=0;
      }
      $table->MaxID($maxID||0,'nocheck');


      if($::dbType eq 'Pg'){
	$seqName=$tblName.'_'.$fldID.'_seq';
	if($maxID){
	  $sql = "select setval ('$seqName',$maxID,true)";
	}else{
	  $sql = "select setval ('$seqName',1,false)";
	}
	unless($::dbh->do($sql)){
	  if ($::DEBUG>1){
	    print STDERR "Error:  Common::pre_restore_tab: Error execute query: $sql\n".
	    "$DBI::errstr\n";
	  }
	}
      }
    }

    $table->StateClean(1);
  }
  return 0;
}

sub oneline_restore($$$){
  my ($table_name, $ptrRow, $ptrTable) = @_;

  return 1 unless (ref($ptrTable)=~/Table/);
  my $clean = $::objDumpDoc->AttrClean;

  my $key = $ptrTable->AttrRule || "$table_name(*).default";
  my $defaultRule=$main::map->get_rule($key);

  my $sql="select * from $table_name";
  my $sth=$::dbh->prepare($sql);
  if ($sth->execute && $sth->rows){
    unless ( $defaultRule == $ASSIGN_CURRENT ){
      &update_query($table_name, $ptrRow);
    }
  }else{
    &insert_query($table_name, $ptrRow);
    $ptrTable->StateClean(0);
  }
    return 0;
}

sub restore_row($$$){
  my ($table_name, $ptrRow, $ptrTable) = @_;
  my ($type,$fldName,$ptrField,$ptrFields,$new,$uid,$fldID,$attrKey,$newValue,$link,$key);

#
# replace IP,neu
#
  return 1 unless (ref($ptrTable)=~/Table/);
  my $clean = $::objDumpDoc->AttrClean;
  my $insert=1;
  my (%changed,$update,$where,$sql,$sth);
  $key = $ptrTable->AttrRule || "$table_name(*).default";
  my $defaultRule=$main::map->get_rule($key);
  my $convTimestamp = ($::dbType eq 'Pg');

  my $defaultUser = $::map->get_rule('user(*).default');
  my $defaultReseller = $::map->get_rule('reseller(*).default');


  if($ptrTable->hasAnyAttr('ip','new','links','buildin') 
     || $convTimestamp  ## check timestamp
    ){
#$ptrTable->AttrIP || $ptrTable->AttrNew || $ptrTable->AttrUID){
    my ($lnkTable,$lnkField,$table,$oldValue);
    $ptrFields=$ptrTable->getFields;
    unless (keys(%{$ptrFields})){
      if ($::DEBUG){
				print STDERR "Error: table '$table_name' has no field\n ";
				die;
      }
    }
    if ( ref( $ptrFields) =~ /HASH/ ) {
      while ( ( $fldName,$ptrField ) = each( %{$ptrFields} ) ) {
				next unless exists($ptrRow->{$fldName});
				next unless (ref($ptrField) =~ /HASH/);
				$oldValue = $ptrRow->{$fldName};
				$key = "$table_name($oldValue).$fldName";
				if ( $newValue = $::map->get_attr( $key ) ) {
					$ptrRow->{$fldName} = $newValue;
					$changed{$fldName} = 1;
					next;
				}
# replace ip
				if ( $type = $ptrField->{'type'} ) {
					if ( $type eq 'ip' ) {
						$ptrRow->{$fldName} = $main::objDumpDoc->replaceIP( $oldValue );
					} elsif ( $type eq 'url' ) {
						if ( $newValue = &Modules::Restore::Tables::Table_domains::getNewDomain( $oldValue ) ) {
							$ptrRow->{$fldName} = $newValue;
							$changed{$fldName} = 1;
						}
					} elsif ( $type eq 'T' && $convTimestamp ) {
						if ( $ptrRow->{$fldName} =~ /^\d+$/){  ## convert '20040130124532' -> '2004-01-30 12:45:32'
							my @chunks = unpack('A4A2A2A2A2A2',$ptrRow->{$fldName});
							$ptrRow->{$fldName} = join( '-', @chunks[0..2] );
							if ( @chunks[3..5] ) {
								$ptrRow->{$fldName}.=' '.join(':',@chunks[3..5]);
							}
						}
					}
				}
				if ( $newValue = $ptrField->{'buildin'} ) {
					if ( $ptrRow->{$fldName} eq $newValue ) {
						$sql = "select $fldName from $table_name where $fldName='$newValue'";
						$sth = $::dbh->prepare($sql);
						if ( $sth->execute ) {
							if ( $sth->rows ) {
								$update = 1;
								$where = "$fldName = '$newValue'";
							}
							$sth->finish;
						}
					}
				}
				if ( $link = $ptrField->{'link'} ) {
					if ( $link =~ /anbieter\.anbieter/ ) {
						$key = "reseller($oldValue).name";
						if ( $newValue = $main::map->get_attr( $key ) ) {
							$ptrRow->{$fldName} = $newValue;
							$changed{$fldName} = 1;
						}
						if ( $oldValue eq 'res0' ) {
							
							if ( $defaultReseller == $ASSIGN_CURRENT ) {
								return 0; # skip for build-in reseller
							}
							if ( $defaultReseller != $RESTORE_DUMPED ) {
								$update = 1;
								$where .= ' AND ' if $where;
								$where .= "($fldName='res0')";
							}
						}
					} elsif ( $link =~ /kunden\.kunde/ ) {
						$key = "user($oldValue).name";
						if ( $newValue = $main::map->get_attr( $key ) ) {
							$ptrRow->{$fldName} = $newValue;
							$changed{$fldName} = 1;
						}
						if ( $oldValue eq $oldUser0 ) {
							if ( $defaultUser == $ASSIGN_CURRENT ) {
								return 0; # skip for build-in user
							}
							if ( $defaultUser != $RESTORE_DUMPED ) {
								$update = 1;
								$where .= ' AND ' if $where;
								$where .= "($fldName='$Modules::Restore::Tables::Table_kunden::newUser0')";
							}
						}
					} elsif ( $link =~ /domains\.domain/ ) {
						$key = "domain($oldValue).name";
						if ( $newValue = $main::map->get_attr( $key ) ) {
							$ptrRow->{$fldName}=$newValue;
							$changed{$fldName} = 1;
						} else {
							print STDERR "Tables::Common : Error: Unable to restore '$table_name'. ".
								"The new domain is not found. Old domain: '$oldValue'\n";
						}
					} elsif ( $link =~ /domains\.id/ ) {
						$key = "domain($oldValue).id";
						if ( $newValue = $main::map->get_attr( $key ) ) {
							$ptrRow->{$fldName} = $newValue;
							$changed{$fldName} = 1;
						} else {
							unless( $clean ) {
								print STDERR "Tables::Common : Error: Unable to restore '$table_name'. ".
									"The new domain-id  is not found for domain-id: #$oldValue\n";
								$insert = 0;
								last;
							}
						}
					} else {
						($lnkTable,$lnkField) = split(/\./,$link);
						$table = $main::objDumpConf->getTable( $lnkTable );
						if ( $lnkField && ( ref($table) =~ /Table/ ) ) {
							if ( $newValue = $table->getMappedValue( $lnkField,$oldValue ) ) {
								$ptrRow->{$fldName} = $newValue;
								$changed{$fldName} = 1;
							} else {
#
# error: no linked field
#
								if ( $::DEBUG ) {
									print STDERR "Tables::Common : Error: Unable to map linked field: '$lnkField', old value: '$oldValue'. Record is skiped\n"
								}
								$insert=0;
								last;
							}
						}
					}
				}

# emulate new state
				if ( $new=$ptrField->{'new'} ) {
					$ptrRow->{$fldName} = $new;
					$changed{$fldName} = 1;
				}
      }
    }
  }
#
# /replace
#
  my ($fldsUnique,$fld,@vals,$firstField,$firstVal,$val,$flds,$oldVal);
  my ($oldIdValue,$newIdValue);

  if ( $update ) {
    $insert = 0;
    if ( $fldID = $ptrTable->AttrID ) {
      $oldIdValue = $ptrRow->{$fldID};
      $where = "($fldID='$oldIdValue')";
      $newIdValue = $oldIdValue;
    } elsif ( $fldsUnique=$ptrTable->Attribute('unique') ) {
      $fldsUnique =~ s/,$//;
      my @fldSet = split(/,/,$fldsUnique);
      foreach $fld ( @fldSet ) {
				$val = $ptrRow->{$fld};
				next unless defined $val; ## null-value
				unless ( $firstField ) {
					$where = "($fld='$val')";
					last;
				}
      }
    } elsif ( $where ) {
      $sql = "select count(*) from $table_name where $where";
      $sth = $main::dbh->prepare($sql);
      if ( $sth->execute && $sth->rows ) {
				($val) = $sth->fetchrow;
				if ( $val>1 ) {
					$update = 0;
					print STDERR "Error:Common::restore_row: Unable to update table '$table_name' for $where\n";
					$where = '';
				}
				$sth->finish;
      }
    }
  }

  my ($newID);

#
# ID field
#
  if ( $fldID = $ptrTable->AttrID() ) {
    if ( $clean ) {
      $oldIdValue = $ptrRow->{$fldID};
      if ($ptrTable->TypeID =~ /^I/ ) {
# calc new ID
				$newIdValue = $oldIdValue + ($ptrTable->MaxID||0);
      } else {
				$newIdValue = $oldIdValue;
      }
    } else {
      if ( $insert ) {
				$oldIdValue = $ptrRow->{$fldID};
				if ( $changed{$fldID} ) {
					$newIdValue = $oldIdValue;
				} else {
					$key="$table_name($oldIdValue).id";
# try find in map
					unless ( $newIdValue = $main::map->get_attr( $key ) ) {
						if ( $ptrTable->TypeID=~/^I/ ) {
# calc new ID if it is numeric
							$newIdValue = $oldIdValue + ($ptrTable->MaxID||0);
						}
					}
				}

				unless ($newIdValue){
					if ( $oldIdValue =~ /^res\d+$/ ) {
						$newIdValue = $main::map->get_attr( 'reseller($oldIdValue).name' );
					} elsif ( $oldUserPrefix &&  ( $oldIdValue =~ /^$oldUserPrefix\d+$/ ) ) {
						$newIdValue = $main::map->get_attr('user($oldIdValue).name');
					}
					$newIdValue = $oldIdValue unless $newIdValue;
				}
  
#
# check duplicated id
#
				$sql = "SELECT $fldID FROM $table_name WHERE $fldID = ?";
				$sth = $main::dbh->prepare( $sql );
				if ( $sth->execute( $newIdValue ) && $sth->rows ) {
      
					delete $ptrRow->{$fldID};
					if ( $defaultRule == $RESTORE_DUMPED){
						$where = "($fldID='$oldIdValue')";
						$update=1;
					} elsif ( $defaultRule == $APPEND_DUMPED ) {
						if ( $newIdValue = $ptrTable->getNextID ) {
#
# register change
#
							$main::map->add_attr($key,$newIdValue);
	
							$ptrRow->{$fldID} = $newIdValue;
							$changed{$fldID} = 1;
						} else {
							print STDERR "Tables::Common: Error:".
								" Can't get new id-value for table '$table_name'\n".
									"Record '$oldIdValue' is skiped\n";
							$insert = 0;
						}  
					} else {
						if ( $defaultRule == $ASSIGN_CURRENT ) {
							print STDERR "Tables::Common: Error:".
								" Can't insert duplicate id (rule: ASSIGN_CURRENT)\n".
									"table: '$table_name', id: '$newIdValue'('$oldIdValue')\n";
						}
						$insert = 0; ## duplicate id
					}
	  
				}
				$sth->finish;
			}
    }
    if ( defined( $oldIdValue ) && defined( $newIdValue ) ) {
      if ( $oldIdValue ne $newIdValue ) {
				$ptrRow->{$fldID} = $newIdValue;
				$key="$table_name($oldIdValue).id";
				$main::map->add_attr($key,$newIdValue);
      }
      $ptrTable->addToMap($fldID,$oldIdValue,$newIdValue);
    }
  }
    
#
# /ID field
#

#
# unique
#
  unless($clean){
    if ($fldsUnique=$ptrTable->Attribute('unique')){
      $fldsUnique=~s/,$//;
      my @fldSet= split(/,/,$fldsUnique);
      $flds='';
      $where='';
      foreach $fld (@fldSet){
				$val = $ptrRow->{$fld};
				next unless defined $val; ## null-value
				unless ($firstField){
					$firstField=$fld;
					$firstVal=$val;
				}
				$flds.="$fld,";
				$where.=" ($fld=?) AND";
				push @vals,$val;
      }
#     undef @fldSet;

      if ($firstField) {
				$where =~ s/AND$//i;
				$flds =~ s/,$//;
				$sql = "SELECT $flds FROM $table_name WHERE $where";
				
				$sth=$::dbh->prepare( $sql );
				if ( $sth->execute( @vals ) ) {
					if ( $sth->rows ) {
						$insert = 0;
						if ($defaultRule == $RESTORE_DUMPED){
							$update = 1;
							$fld = $fldSet[0];
							$where = "$firstField='$firstVal'";
						} else {
							$update = 0;
							unless ( $defaultRule == $ASSIGN_CURRENT){
								print STDERR "Error: Modules::Restore::Tables::Common:\n".
									"Can't insert duplicated unique field(s) into table $table_name ($fldsUnique) values: ".
										join(',',@vals)."\n";
							}
						}
					}
					$sth->finish;
				}
      }
    }
  }
#
# /unique
#
  if($insert){

    &insert_query($table_name, $ptrRow);
    $ptrTable->StateClean(0);

  }elsif ($update){

    &update_query($table_name, $ptrRow, $where);

  }
  return 0;
}

sub pre_restore_tab($$$$) {
  my ($table_name, $ptrConfXML,$clean,$xmlTable) = @_;
  unless ($table_name && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Modules::Restore::Tables::Common::pre_restore_tab : wrong arguments ($table_name,..)\n";
    return 1;
  }

#  print "pre restore table '$table_name'\n" if $::DEBUG;
  my ($table,$tblName,$tbl,$key);
  return 1 unless $table=$ptrConfXML->getTable($table_name);
  return 1 unless (ref($table)=~/Table/);

  if($table->Attribute('insert')){
#
# expand attribute 'insert' 
#
    foreach $tblName ($ptrConfXML->getTablesList){
      $tbl = $ptrConfXML->getTable($tblName);
      next unless ( ref($tbl) =~ /Table/ );
      if ( ( !$tbl->Attribute('insert') ) && $tbl->getLinks( $table_name ) ) {
				$key = $tbl->AttrRule || $tblName.'(*).default';
				$main::map->add_rule( $key, $APPEND_DUMPED );
				$tbl->Attribute( 'insert', 1 );
      }
    }
  }


#
# calc maximum of id
#
  my ($fldName,$sql,$sth,$maxID,$defaultRule);
  if ( ( $table->TypeID=~/^I/ ) && ( $fldName = $table->AttrID ) ) {
    $key = $table->AttrRule || $table_name.'(*).default';
    unless ( $defaultRule = $main::map->get_rule( $key ) ) {
      $main::map->add_rule($key,$APPEND_DUMPED);
    }
    
    $sql = "SELECT MAX($fldName) FROM $table_name";
    $sth = $main::dbh->prepare( $sql );
    if ( $sth->execute ) {
      ($maxID) = $sth->fetchrow if $sth->rows;
      $sth->finish;
    }
    $table->MaxID($maxID || 0);
  }

  return 0;

}

sub post_restore_tab($$$$) {
  my ($tblName,$ptrConfXML,$clean,$xmlTable)=@_;

  unless ($tblName && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Common::post_restore_tab : wrong arguments ($tblName,..)\n";
    return 1;
  }

#  print "pre restore table '$table_name'\n" if $::DEBUG;
  my ($table,$sth);
  unless (($table=$ptrConfXML->getTable($tblName)) && (ref($table)=~/Table/)){
    if($::DEBUG){
      print STDERR "Warning: Common::post_restore_row: Table '$tblName' not found\n";
    }
    return 1;
  }

  my ($fldID,$sql,$maxID,$seqName);

  if (($table->TypeID=~/^I/) && ($fldID=$table->AttrID)){
    unless($maxID=$table->MaxID){
      $sql = "select max($fldID) from $tblName";
      $sth=$::dbh->prepare($sql);
      if($sth->execute && $sth->rows){
	($maxID)=$sth->fetchrow;
	$sth->finish;
	$table->MaxID($maxID || 0);
      }
    }
    

    if($::dbType eq 'Pg'){
      $seqName=$tblName.'_'.$fldID.'_seq';
      if($maxID){
	$sql = "select setval ('$seqName',$maxID,true)";
      }else{
	$sql = "select setval ('$seqName',1,false)";
      }
      unless($::dbh->do($sql)){
	if ($::DEBUG>1){
	  print STDERR "Error:  Common::post_restore_tab: Error execute query: $sql\n".
	    "$DBI::errstr\n";
	}
      }
    }
  }

  return 0;
}

#
#
#
# Mapping
#
#
#


sub pre_map_tab($$$$) {
  my ($table_name, $ptrConfXML,$clean,$xmlTable) = @_;
  unless ($table_name && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Modules::Restore::Tables::Common::pre_map_tab : wrong arguments ($table_name,..)\n";
    return 1;
  }
  my $ptrTable=$ptrConfXML->getTable($table_name);
  return 1 unless (ref($ptrTable)=~/Table/);


  my ($fldID,$link,$lnkTable,$lnkField,$tbl,$sql,$sth,$maxID);

  if( ($fldID=$ptrTable->AttrID()) && 
      ($ptrTable->getFieldAttr($fldID,'type')!~/^I$/)) {
# ID field
    $sql = "select max($fldID) from $table_name";
    $sth=$::dbh->prepare($sql);
    if($sth->execute && $sth->rows){
      ($maxID)=$sth->fetchrow;
      $sth->finish;
      $ptrTable->MaxID($maxID || 0);
    }

    if($link=$ptrTable->getFieldAttr($fldID,'link')){
# is link
      unless (exists($RsrvdLink{$link})){
# is not reserved link
	($lnkTable,$lnkField)=split(/\./,$link);
	$tbl=$ptrConfXML->getTable($lnkTable);
	next unless (ref($tbl)=~/Table/);
	$ptrTable->Attribute('mapID',$lnkField);
      }
    }else{
      $ptrTable->Attribute('mapID','id');
      my $key=$ptrTable->AttrRule || $table_name.'(*).default';

      unless ($main::map->get_rule($key)){
#
# default rule for the key field
#
	if ($clean){
	  $main::map->add_rule($key,$RESTORE_DUMPED);
	}else{
	  $main::map->add_rule($key,$ASSIGN_CURRENT);
	}
      }
    }
  }
#
# calculate old user prefix
#
  unless ($oldUserPrefix){
    $oldUserPrefix=$Modules::Restore::Tables::Table_admin::oldUserPrefix ||
      $main::map->oldUserPrefix ;
  }



}

sub map_row($$$){
  my ($table_name, $ptrRow, $ptrTable) = @_;
  my ($fldID,$mapID,$key,$newDomain,$oldID);

  return 0 if $::objDumpDoc->AttrClean;

  if( ($mapID=$ptrTable->Attribute('mapID')) && ($fldID=$ptrTable->AttrID())){
    $oldID=$ptrRow->{$fldID};

#
# check link to kunden.kunde or anbieter.anbieter
#
    unless ($oldID=~/^res\d+$/ || ($oldUserPrefix && $oldID=~/^$oldUserPrefix\d+$/)){
      $key="$table_name($oldID).id";
# register answer
      $main::map->add_attr($key,'');
    }
  }
  if($ptrTable->Attribute('unique')){
    my $tblDomains;
    my $ptrFields=$ptrTable->getFields;
    my ($lnkTable,$lnkField,$table,$oldValue,$unique,$sql,$sth,$cnt,$fldName,$ptrField);
 
   if(ref($ptrFields)=~/HASH/){
      while (($fldName,$ptrField) = each(%{$ptrFields})){
	next unless exists($ptrRow->{$fldName});
	next unless (ref($ptrField)=~/HASH/);
	next unless ($ptrField->{'unique'});
# unique field
	next unless exists $ptrRow->{$fldName}; ## null value

	if ($ptrField->{'buildin'}){
	  next if $ptrField->{'buildin'} eq $oldValue;
	}

	$oldValue=$ptrRow->{$fldName};
	if (($ptrField->{'type'} eq 'url') ){
	  $newDomain=&Modules::Restore::Tables::Table_domains::getNewDomain($oldValue); # Table_domains
	  $oldValue=$newDomain if $newDomain;
	}


	$sql="select $fldName from $table_name where $fldName=?";
	$sth=$::dbh->prepare($sql);
	if ($sth->execute($oldValue) &&  $sth->rows){
	  $key="$table_name($oldValue).$fldName";
	  $::map->add_attr($key,'');
	}
	$sth->finish;
      }
    }
  }

  if($key){
    $::map->add_rule("$table_name(*).default",$ASSIGN_CURRENT);
  }

}

sub nothing {
  return 0;
}

1;

