package Modules::Restore::Tables::Table_admin;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####

use XML::DOM;
use XML::DOM::NodeList;

use Modules::Restore::Map;
use Modules::Restore::Service;
use Modules::Restore::Table;
use Modules::Restore::DumpConfXML;
use Modules::Restore::DumpDoc;

use Modules::Restore::Tables::Common;
use Modules::Restore::Tables::Common qw(&update_query($$$) &insert_query($$));

use vars qw( $ptrAdmin $oldUserPrefix $newUserPrefix);


BEGIN {
  use Exporter;
  @ISA = 'Exporter';
  @EXPORT = qw( &pre_tab($$$$)  &post_restore_tab($$$$)
	  &restore_row($$$) &map_row($$$) &getAdminValue($)
	      $ptrAdmin $oldUserPrefix $newUserPrefix);

  $ptrAdmin=undef;
  $oldUserPrefix='';
  $newUserPrefix='';
}

use strict;

sub pre_tab($$$$) {
  my ($tblName,$ptrConfXML,$clean,$xmlTable)=@_;
  unless ($tblName && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Modules::Restore::Tables::Table_admin::pre_tab : wrong arguments ($tblName,..)\n";
    return 1;
  }
#  print STDERR "pre-phase table 'admin'\n" if $::DEBUG;

  my $sth = $main::dbh->prepare("select * from admin");
  $sth->execute();
  $ptrAdmin = $sth->fetchrow_hashref;
  $sth->finish;
#
# read states of services
#  
  my ($srvName,$service,$flag,$field,$sql,$state,$table,$fldName);
  foreach $srvName ($ptrConfXML->getServicesList){
    $service = $ptrConfXML->getService($srvName);
    next unless(ref($service)=~/Service/);
    $flag=$service->Flag;
    next unless(ref($flag)=~/Flag/);
    $field=$flag->Field;
    next unless(ref($field)=~/Field/);
    if (($tblName=$field->Table) eq 'admin') {
      $state=$ptrAdmin->{$field->Field};
    }else{
      next unless ($table=$ptrConfXML->getTable($tblName)); # check if has table
      next unless (ref($table)=~/Table/);
      $fldName=$field->Field;
      next unless ($table->getFieldAttrSet($fldName)); # check if has field
      $sql="select max($fldName) from $tblName";
      $sth = $main::dbh->prepare($sql);
      $sth->execute();
      $state=($sth->fetchrow)[0];
      $sth->finish;
    }
    $service->State($state);
  }
  $newUserPrefix=$ptrAdmin->{'userprefix'} || $::user_prefix ;
  $oldUserPrefix=$main::map->oldUserPrefix;
  if ( $newUserPrefix && $oldUserPrefix){
    $main::map->add_attr("user($oldUserPrefix).prefix",$newUserPrefix);
  }

  return 0;
}

sub getAdminValue($){
  my $fldName=shift;
  return undef unless (ref($ptrAdmin)=~/HASH/);
  return $ptrAdmin->{$fldName};
}

sub post_restore_tab($$$$) {
  my ($tblName,$ptrConfXML,$clean,$xmlTable)=@_;

  print STDERR "post restore table 'admin'\n" if $::DEBUG;

  return 0 unless (ref($ptrConfXML)=~/DumpConfXML/);

  my $ptrServ=$main::map->getAttrHash('service(*).state');
  my ($key,$value,$srv,$ptrFlds,$flag,$field,$tblField,
      $sql,$fldName,$offVal,$table);

  while (($key,$value)=each(%{$ptrServ})){
    $srv=$ptrConfXML->getService($key);
    next unless(ref($srv)=~/Service/);
    $flag=$srv->Flag;
    next unless(ref($flag)=~/Flag/);
    next if ($value eq $srv->On);
    $ptrFlds=$flag->getFields;
    $offVal=$srv->Off || '0';
    foreach $field (@{$ptrFlds}){
    
      next if (($tblName=$field->Table) eq 'admin');
      next unless ($table=$ptrConfXML->getTable($tblName)); # check if has table
      next unless (ref($table)=~/Table/);
      
      $fldName=$field->Field;
      next unless ($table->getFieldAttrSet($fldName)); # check if has field
      
      $sql="update $tblName set $fldName='$offVal'";
      $main::dbh->do($sql);
    }
  }

  return 0;
}


sub restore_row($$$) {
  my ($table_name, $ptrRow, $ptrTable) = @_;

  # restored fields
  my %template = 
    (
     'login'		=> '',
     'longpw' 		=> '',
     'emailbetreff'	=> '',
     'emailtext'	=> '',
     'emailadresse'	=> '',
     'emailname'	=> '',
     'ftpmaxkb'		=> '',
     'popmaxkb'		=> '',
     'indexcode'	=> '',
     'indexfile'	=> '',
     'language'		=> '',
     'ispemail'		=> '',
     'ispservername'	=> '',
     'isplastrun'	=> '',
     'httpdspezial'	=> '',
    );
	
  my $sth = $main::dbh->prepare("SELECT * FROM admin");
  $sth->execute
    or die("Error #0101: Can not restore 'admin' table: $DBI::errstr");
  my $href_admin = $sth->fetchrow_hashref;

  foreach my $attr (keys %template) {
    $template{$attr} = $ptrRow->{$attr};
  }

  print STDERR "attr: ".$main::map->get_rule('admin(*).dnstemplate') .
    "\tconst:". $RESTORE_DUMPED."\n" if ($::DEBUG);
  if ($main::map->get_rule('admin(*).dnstemplate') == $RESTORE_DUMPED) {
    $template{'dnstemplate'} = $ptrRow->{'dnstemplate'};
  }
  my $key='domain('.$ptrRow->{'standarddomain'}.').name';
  if ($main::map->get_rule('domain(*).standard') == $RESTORE_DUMPED) {
    $template{'standarddomain'} = $ptrRow->{'standarddomain'};
    $::map->add_attr($key,$ptrRow->{'standarddomain'});
  }else{
    $::map->add_attr($key,$href_admin->{'standarddomain'});
  }

  $template{'standardip'}=$main::objDumpDoc->replaceIP($ptrRow->{'standardip'});

  &Modules::Restore::Tables::Common::update_query('admin', \%template);


#
# register change of the service state
#
  my ($srvName,$srv,$flag,$field,$sql,$table,$state,$newState,$dumpConf);
  $dumpConf=$main::objDumpDoc->DumpConfXML;

  return 0 unless (ref($dumpConf)=~/DumpConfXML/);

  foreach $srvName ($dumpConf->getServicesList){
    $srv=$dumpConf->getService($srvName);
    next unless(ref($srv)=~/Service/);
    $flag=$srv->Flag;
    next unless(ref($flag)=~/Flag/);
    $field=$flag->Field;
    next unless(ref($field)=~/Field/);
    if (($table=$field->Table) eq 'admin') {
      $state=$ptrRow->{$field->Field};
      $newState=$srv->State;
      if ($state ne $newState){
	$key="service(".$srv->Name.").state";
	$main::map->add_attr($key,$newState);
      }
    }
  }
  return 0;

}

sub map_row($$$) {
  my ($table_name, $ptrRow, $ptrTable) = @_;

#  print STDERR "processing admin record\n" if ($::DEBUG);
  my ($oldIP,$oldDom,$oldValue);
  if ($oldIP = $ptrRow->{'standardip'}){
      $main::map->add_attr("ip($oldIP).address",$ptrAdmin->{'standardip'});
  }
  if ($oldDom = $ptrRow->{'standarddomain'}){
    my $newDom =$ptrAdmin->{'standarddomain'};
    $main::map->add_attr("domain($oldDom).standard",$newDom);
    my $tblDomains=$::objDumpConf->getTable('domains');
    if (ref($tblDomains)=~/Table/){
      $tblDomains->addToMap('domain',$oldDom,$newDom)
    }
  }
  if ($oldValue = $ptrRow->{'userprefix'}){
    $oldUserPrefix = $oldValue;
    $main::map->add_attr("user($oldValue).prefix",$ptrAdmin->{'userprefix'}||$newUserPrefix);
  }
 
#
# read states of services
#  
  my ($srvName,$service,$flag,$field,$sql,$tblName,$state,$table,$fldName);
  foreach $srvName ($main::objDumpConf->getServicesList){
    $service = $main::objDumpConf->getService($srvName);
    next unless(ref($service)=~/Service/);
    $flag=$service->Flag;
    next unless(ref($flag)=~/Flag/);
    $field=$flag->Field;
    next unless(ref($field)=~/Field/);
    if (($tblName=$field->Table) eq 'admin') {
      $state=$ptrRow->{$field->Field};
      if (($state ne $service->State) && $service->isStateOff){
	$main::map->add_attr("service($srvName).state",$service->State);
      }
    }
  }
  return 0;
}

END {
  $ptrAdmin=undef;
}

1;
