package Modules::Restore::Tables::Table_cstm_skins;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####


BEGIN {
  use Exporter;
  @ISA = qw(Exporter);
  @EXPORT = qw(&restore_row($$$) &pre_restore_tab($$$$) &clean_tab($$));	
}

use XML::DOM;
use XML::DOM::NodeList;

use strict;

use Modules::Restore::Map;
use Modules::Restore::Table;
use Modules::Restore::DumpConfXML;
use Modules::Restore::DumpDoc;

use Modules::Restore::Tables::Common;
use Modules::Restore::Tables::Common qw(&update_query($$$) &insert_query($$));

my %preinstSkins;

sub clean_tab($$){
  my ($table_name, $ptrConfXML) = @_;
  unless ($table_name && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Table_cstm_skinss::clean_tab : wrong arguments ($table_name,..)\n";
    return 1;
  }
  my ($table,$id);
  unless($table=$ptrConfXML->getTable($table_name)){
    return 1;
  }

#
# clean tables
#
  my @deletedIDs;
  my $sql = "select id,path from $table_name where not (owner_type in ('admin','preinstalled'))";
  my $sth=$::dbh->prepare($sql);
  my ($path);
  if($sth->execute && $sth->rows){
    while (($id,$path)=$sth->fetchrow){
      push @deletedIDs,$id;

      $path='/'.$path unless $path=~/^\//;
      next unless (()=$path=~/(\/)/g);
      $path = $::confixx_htmlDir.$path;
      if(-d $path){
	unless (system ("rm -rf $path")==0){ 
	  print STDERR "Error: unable to delete dir: $path\n";
	}
      }
    }
    $sth->finish;
  }

  if(@deletedIDs){

    $sql = "delete from $table_name where not (owner_type in ('admin','preinstalled'))";
    unless($::dbh->do($sql)){
      print STDERR "Error: Table_cstm_skins::clean_tab: unable to execute: $sql\n".
	$::dbh->errstr."\n";
      return 0;
    }
    $table->StateClean(1);

#
# set to default skin
#
    $sql="update skins set skin_id=0 where skin_id in (".join(',',@deletedIDs).")";
    unless($::dbh->do($sql)){
      print STDERR "Error: Table_cstm_skins::clean_tab: unable to execute: $sql\n".
	$::dbh->errstr."\n";
    }
  }

  return 0;
}


sub pre_restore_tab($$$$) {
  my ($table_name, $ptrConfXML,$clean,$xmlTable) = @_;
  unless ($table_name && ref($ptrConfXML)=~/DumpConfXML/){
    print STDERR "Modules::Restore::Tables::Common::pre_restore_tab : wrong arguments ($table_name,..)\n";
    return 1;
  }

#  print "pre restore table '$table_name'\n" if $::DEBUG;
  my ($table,$tblName,$tbl);
  return 1 unless $table=$ptrConfXML->getTable($table_name);
  return 1 unless (ref($table)=~/Table/);
  my $key="$table_name(*).default";

  if($table->Attribute('insert')){
#
# expand attribute 'insert' 
#
    $main::map->add_rule($key,$APPEND_DUMPED);
    foreach $tblName ($ptrConfXML->getTablesList){
      $tbl=$ptrConfXML->getTable($tblName);
      next unless (ref($tbl)=~/Table/);
      if ((!$tbl->Attribute('insert')) && $tbl->getLinks($table_name)){
	$tbl->Attribute('insert',1);
      }
    }
  }
#
# calc maximum of id
#
  my $sql;
  $sql="select max(id) from $table_name";

  if($clean){
    $sql.=" where owner_type='preinstalled'";
  }
  my $sth=$main::dbh->prepare($sql);
  $sth->execute;
    
  my($maxID)=$sth->fetchrow if $sth->rows;
  $sth->finish;
  $table->MaxID($maxID || 9); ## build-in skins
  

  my ($id,$path);
  $sql="select id,path from custom_skins where owner_type='preinstalled'";
  $sth=$main::dbh->prepare($sql);
  $sth->execute;
  %preinstSkins=();

  $key='skins(*).default';
  my $restoreDumped = ($::map->get_rule($key)==$RESTORE_DUMPED);

  while (($id,$path)=$sth->fetchrow){
    $preinstSkins{$id}=$path;
    if($restoreDumped){
      $table->addToMap('path',$path,$path);
      $::map->add_attr("$table_name($path).path",$path);
      $table->addToMap('id',$id,$id);
      $::map->add_attr("$table_name($id).id",$id);
    }
  }
  $sth->finish;
  return 0; 
}

sub restore_row($$$){
  my ($table_name, $ptrRow, $ptrTable) = @_;

  my $key='skins(*).default';
  my $restoreDumped = ($::map->get_rule($key)==$RESTORE_DUMPED);
  my $clean = $::objDumpDoc->AttrClean;

  my ($oldPath,$newPath,$oldId,$oldRes,$newRes,$newId);
  $oldPath=$ptrRow->{'path'};
  $oldId=$ptrRow->{'id'};

  if ($ptrRow->{'owner_type'} =~/preinstalled/){
    if ($restoreDumped && ($newPath=$preinstSkins{$oldId})){ 
      $::map->add_attr("$table_name($oldPath).path",$newPath);
      $::map->add_attr("$table_name($oldId).id",$oldId);
      $ptrTable->addToMap('path',$oldPath,$newPath);
      $ptrTable->addToMap('id',$oldId,$oldId);
    }
    unless(exists($preinstSkins{$oldId})){
      $::map->add_attr("$table_name($oldPath).path",$oldPath);
      $::map->add_attr("$table_name($oldId).id",$oldId);
      $ptrTable->addToMap('path',$oldPath,$oldPath);
      $ptrTable->addToMap('id',$oldId,$oldId);
      
      &Modules::Restore::Tables::Common::insert_query($table_name,$ptrRow);
    }
  }else{
    if ($clean){
#
# register changes
#      
      $::map->add_attr("$table_name($oldId).id",$oldId);
      $ptrTable->addToMap('id',$oldId,$oldId);
      $::map->add_attr("$table_name($oldPath).path",$oldPath);
      $ptrTable->addToMap('path',$oldPath,$oldPath);

    }else{
      $oldRes = $ptrRow->{'anbieter'};
      if ($newRes = $::map->get_attr("reseller($oldRes).name")){
	$ptrRow->{'anbieter'}=$newRes;
      }
      $newId = $ptrTable->getNextID;
      $::map->add_attr("$table_name($oldId).id",$newId);
      $ptrTable->addToMap('id',$oldId,$newId);

      $ptrRow->{'id'}=$newId;
      $newPath=$oldPath;
      $newPath=~s/_(\d+)$/$newId/;
      $::map->add_attr("$table_name($oldPath).path",$newPath);
      $ptrTable->addToMap('path',$oldPath,$newPath);
      $ptrRow->{'path'}=$newPath;
    }

    &Modules::Restore::Tables::Common::insert_query($table_name,$ptrRow);

  }
  return 0;
}

1;
