package Modules::Restore::Tables::Table_domains;

########## Confixx(R) 3.0 Professional ############
####### Copyright SWsoft, Inc. 2004-2005 ##########
#### http://www.sw-soft.com - info@sw-soft.com ####



use XML::DOM;
use XML::DOM::NodeList;

use vars qw(%oldDomains $tblDomains $minSuffix $minSuffixLen);

BEGIN {
  use Exporter;
  @ISA = qw(Exporter);
  @EXPORT = qw(&clean_tab($$) &restore_row($$$) &pre_restore_tab($$$$) 
	       &post_restore_tab($$$$) &getNewDomain($));	

  %oldDomains=();
  $tblDomains=undef;
  $minSuffix='';
  $minSuffixLen='';

}

use strict;

use Modules::Restore::Map;
use Modules::Restore::Table;
use Modules::Restore::DumpConfXML;
use Modules::Restore::DumpDoc;

use Modules::Restore::Tables::Common;
use Modules::Restore::Tables::Table_admin;
use Modules::Restore::Tables::Table_kunden;

use Modules::Restore::Tables::Table_admin qw(&getAdminValue($));
use Modules::Restore::Tables::Common qw(&update_query($$$) &insert_query($$));

use DBI;

sub clean_tab($$){
  my ($table_name, $ptrConfXML) = @_;
  return 1;
}

sub restore_row($$$){
  my ($table_name, $ptrRow, $ptrTable) = @_;
  my ($oldDomain,$key,$newDomain,$domType,$oldKunde,$newKunde,$newRes,$oldRes,
      $sth,$newID,$oldID,$update,$insert,$where,$oldVersion);

  unless(ref($ptrTable)=~/Table/){
    print ERRSTD "Table_domains.pm(restore_row) : error wrong arguments\n";
    return 1;
  }
  $oldDomain = $ptrRow->{'domain'};
  $where = "domain='$oldDomain'";
  $oldKunde = $ptrRow->{'kunde'};
  if (exists($ptrRow->{'id'})){
    $oldID =$ptrRow->{'id'};
  }else{
    $oldVersion = 2;
  }
  if($newKunde = $::map->get_attr("user($oldKunde).name")){
    $ptrRow->{'kunde'}=$newKunde;
  }
  $oldRes=$ptrRow->{'anbieter'};
  if ($newRes = $::map->get_attr("reseller($oldRes).name")){
    $ptrRow->{'anbieter'}=$newRes;
  }
  my $defaultRule=$main::map->get_rule('domain(*).default');
  my $restoreDumped=($defaultRule == $RESTORE_DUMPED);
  $update=1 if ($restoreDumped);
  if ($newKunde eq $Modules::Restore::Tables::Table_kunden::newUser0){
    ## reserved user name
    unless ($restoreDumped || $::objDumpDoc->AttrClean) {
      return 0; ## skipp
    }
  }
  $domType=$ptrRow->{'richtigedomain'};
  $key="domain($oldDomain).name";
  unless($newDomain = $main::map->get_attr($key)){
    if ($domType == 3 || $domType == 2){
      $newDomain=&getNewDomain($oldDomain);
    }else{
      $newDomain=&getNewDomain($oldDomain,'suffix') || $oldDomain;
    }
  }

  if($newDomain){
    
    unless($oldVersion){
      $oldID=$ptrRow->{'id'};
      delete $ptrRow->{'id'};
    }
    $sth=$::dbh->prepare("select id from domains where domain='$newDomain'");
    $sth->execute;
    if ($sth->rows){
      ($newID)=$sth->fetchrow;
    }else{
      $newID=0;
    }
    $sth->finish;

    if($update && $newID){

      &Modules::Restore::Tables::Common::update_query('domains',$ptrRow,"id=$newID");

      $::map->add_attr("domain($oldDomain).name",$newDomain);
      $ptrTable->addToMap('domain',$oldDomain,$newDomain);
      unless($oldVersion){
	$::map->add_attr("domain($oldID).id",$newID);
	$ptrTable->addToMap('id',$oldID,$newID);
      }
      return 0;
    }

    unless ($newID){
      $ptrRow->{'domain'}= $newDomain;

      &Modules::Restore::Tables::Common::insert_query('domains',$ptrRow);

      $::map->add_attr("domain($oldDomain).name",$newDomain);
      $ptrTable->addToMap('domain',$oldDomain,$newDomain);

      $sth=$::dbh->prepare("select id from domains where domain='$newDomain'");
      $sth->execute;
      ($newID)=$sth->fetchrow;
      $sth->finish;
      if ($newID){
	unless($oldVersion){
	  $ptrTable->addToMap('id',$oldID,$newID);
	  $::map->add_attr("domain($oldID).id",$newID);
	}
	$ptrTable->MaxID($newID);
      }

    }else{
      if ($defaultRule == $APPEND_DUMPED){
	print STDERR "Error: there are domain's name: $newDomain\n";
      }
    }
  }

  return 0;
}

sub pre_restore_tab($$$$) {
  my ($table_name, $ptrConfXML,$clean,$xmlTable) = @_;

#  print STDERR "pre restore table 'domains'\n" if $::DEBUG;

  unless ((ref($xmlTable)=~/XML/) && (ref($ptrConfXML)=~/DumpConfXML/)){
    print ERRSTD "Tables_domains.pm : error wrong types of arguments\n";
    return 1;
  }
  my ($kid,$ptrRow,$domType,$table,$sql,$sth,$oldName,$newName,$oldKunde,
      $newKunde,$tblName,$key);

  my $newPrefix=$Modules::Restore::Tables::Table_admin::newUserPrefix||$::user_prefix;
  my $newUser0=$newPrefix.'0';

  $tblDomains=$ptrConfXML->getTable('domains');
  unless (ref($tblDomains)=~/Table/){
    print ERRSTD "Tables_domains.pm(pre_restore_tab) : error table 'domains' not found\n";
    return 1;
  }
  my $tblUser =$ptrConfXML->getTable('kunden');
  unless (ref($tblUser)=~/Table/){
    print ERRSTD "Tables_domains.pm(pre_restore_tab) : error table 'kunden' not found\n";
    return 1;
  }

  my (%domains,$dom);
  $clean= $::objDumpDoc->AttrClean;
  my $stdDom = $::stdDomain;
  unless ($stdDom){
    $stdDom=&getAdminValue('standarddomain');
  } 


  if($main::map->get_rule('domain(*).standard') == $RESTORE_DUMPED){
#
# rewrite standard domain
#
    if(my $ptrStdDom=$main::map->getAttrHash('domain(*).standard')){
      while (($oldName,$newName) = each(%{$ptrStdDom})){
	next unless $newName;
	$stdDom = $newName;
	$tblDomains->addToMap('domain',$oldName,$newName);
	$main::map->add_attr("domain($oldName).name",$newName);
      }
    }
  }

  if(my $ptrHash=$main::map->getAttrHash('domain(*).suffix')){
    while (($oldName,$newName) = each(%{$ptrHash})){
      $tblDomains->addToMap('suffix',$oldName,$newName);
    }
  }

  my ($ruleStandard);
  foreach $kid ($xmlTable->getChildNodes()){
    next unless ($kid->getNodeType==ELEMENT_NODE);
    next unless ($kid->getNodeName() eq 'row');
    $ptrRow=$main::objDumpDoc->decode_row($kid);
    next unless (ref($ptrRow)=~/HASH/);
    $domType=$ptrRow->{'richtigedomain'};
    $oldName=$ptrRow->{'domain'};
    $key="domain($oldName).name";
    if($newName=$main::map->get_attr($key)){
      ## check domain name
      if ($newName=~/^(\*\.)?([a-zA-Z0-9\-]+\.)+[a-zA-Z]+$/){
	$tblDomains->addToMap('domain',$oldName,$newName);
	next;
      }
    }
    if($domType==3){
      $oldKunde=$ptrRow->{'kunde'};
      $newKunde=$main::map->get_attr("user($oldKunde).name");
      $newKunde=$tblUser->getMappedValue('kunde',$oldKunde) unless($newKunde);
      $newKunde=$oldKunde unless($newKunde);

      next unless($newKunde);

      $newName = "$newKunde.$stdDom";

      $main::map->add_attr($key,$newName);
      $tblDomains->addToMap('domain',$oldName,$newName);
    }
  }
  
#  if($tblDomains->Attribute('insert')){
#    my $table;
#    foreach my $tblName ($ptrConfXML->getTablesList){
#      $table=$ptrConfXML->getTable($tblName);
#      next unless (ref($table)=~/Table/);
#      if ((!$table->Attribute('insert')) && $table->getLinks('domains')){
#	$table->Attribute('domains',1); ## link to 'domains'
#	$main::map->add_rule("$tblName(*).default",$APPEND_DUMPED);
#	$table->Attribute('insert',1);
#      }
#    }
#  }

  return 0 if ($clean);

# will be maked in common part
#
#  foreach my $tblName ($ptrConfXML->getTablesList){
#    $table=$ptrConfXML->getTable($tblName);
#    next unless (ref($table)=~/Table/);
#    if ($table->getLinks('domains')){
#      $table->Attribute('domains',1); ## link to 'domains'
#      $main::map->add_rule("$tblName(*).default",$APPEND_DUMPED);
#      $table->Attribute('insert',1);
#    }
#  }
#
#
  return 1; ## goto common part
}

sub post_restore_tab($$$$) {
  my ($tblName,$ptrConfXML,$clean,$xmlTable)=@_;

  return 0;
}

#
#
# Mapping
#
#

sub pre_map_tab($$$$) {
  my ($table_name, $ptrConfXML,$clean,$xmlTable) = @_;

  my $key='domain(*).default';

  unless( $main::map->get_rule($key) ){
    if ($clean){
      $main::map->add_rule($key,$RESTORE_DUMPED);
    }else{
      $main::map->add_rule($key,$ASSIGN_CURRENT);
    }
  }

  unless ($clean){
    my $sql="select domain,richtigedomain from domains";
    my $sth=$::dbh->prepare($sql);
    $sth->execute;
    %oldDomains=();
    while (my ($dom,$type)=$sth->fetchrow){
      $oldDomains{$dom}=$type;
    }
  }

 $tblDomains=$ptrConfXML->getTable('domains');
  unless (ref($tblDomains)=~/Table/){
    print ERRSTD "Tables_domains.pm(pre_restore_tab) : error table 'domains' not found\n";
    return 1;
  }


  my $ptrHash=$::map->getAttrHash('domain(*).standard');
  if (ref($ptrHash)=~/HASH/){
    while (my ($key,$val)=each(%{$ptrHash})){
      if ($key){
	$tblDomains->addToMap('domain',$key,$val);
      }
    }
  }
  $minSuffix='';
  $minSuffixLen=0;
  return 0;
}

sub map_row($$$) {
  my ($table_name, $ptrRow, $ptrTable) = @_;

  return 1 unless (ref($ptrRow)=~/HASH/);

  my ($kid,$domType,$oldDomain,$key,$newDomain);
  $domType=$ptrRow->{'richtigedomain'};
  $oldDomain=$ptrRow->{'domain'};
  if ($oldDomain && ($domType != 3) && ($domType != 2) ){
    $key="domain($oldDomain).name";
    if ($newDomain=&getNewDomain($oldDomain)){
      $main::map->add_attr($key,$newDomain);
    }else{
      $newDomain=$oldDomain;
    }
    if (exists($oldDomains{$newDomain})){
      if (exists($oldDomains{$oldDomain})){
	$main::map->add_attr($key,'');
      }else{
	$main::map->add_attr($key,$oldDomain);
      }
    }else{
      $main::map->add_attr($key,$newDomain);
    }
  }

  return 0;
}

sub post_map_tab($$$$) {
  my ($table_name, $ptrConfXML,$clean,$xmlTable) = @_;
  if ($minSuffix && (ref($tblDomains)=~/Table/)){
    if (my $newSuffix = $tblDomains->getMappedValue('domain',$minSuffix)){
      $::map->add_attr("domain($minSuffix).suffix",$newSuffix);
    }
  }
  return 0;
}

sub getNewDomain ($){

  return undef unless (ref($tblDomains)=~/Table/);
  my $oldDomain=shift;
  my $mapName=shift || 'domain';

  my ($prefix,$newDomain,$suffixLen,$newSuffix,@pnts);
  $prefix='';
  my $suffix=$oldDomain;
  while ($suffix){
    if($newSuffix = $tblDomains->getMappedValue($mapName,$suffix)){
      $newDomain="$prefix$newSuffix";
      $tblDomains->addToMap($mapName,$oldDomain,$newDomain);
      $tblDomains->addToMap('domain',$oldDomain,$newDomain) unless ($mapName eq 'domain');
      @pnts =($suffix=~/\./g);
      $suffixLen=scalar(@pnts);
      if(!$minSuffixLen ||($minSuffixLen>$suffixLen)){
	$minSuffix=$suffix;
	$minSuffixLen=$suffixLen;
      }elsif($minSuffixLen &&($minSuffixLen==$suffixLen)){
	if($minSuffix ne $suffix){
	  $::map->add_attr("domain($suffix).suffix",$newSuffix);
	  $minSuffix = $suffix;
	}
      }

      last;
    }
    last unless ($suffix =~ /^([^\.]+\.)(.+)$/);
    $prefix .= $1;
    $suffix = $2;
  }
  return $newDomain;
}

END {
  %oldDomains=();
  $tblDomains=undef;
}

return 1;
